unit backpack;
	{ This unit handles both the inventory display and the }
	{ FieldHQ interface, which uses many of the same things. }
{
	GearHead: Arena, a roguelike mecha CRPG
	Copyright (C) 2005 Joseph Hewitt

	This library is free software; you can redistribute it and/or modify it
	under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation; either version 2.1 of the License, or (at
	your option) any later version.

	The full text of the LGPL can be found in license.txt.

	This library is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
	General Public License for more details. 

	You should have received a copy of the GNU Lesser General Public License
	along with this library; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
}

interface

uses
{$IFDEF PATCH_GH}
	gears_base,
{$ENDIF PATCH_GH}
	gears,locale
{$IFDEF SDLMODE}
	,sdlgfx
{$ENDIF SDLMODE}
	,ui4gh;

const
	TRIGGER_GetItem = 'GET';

{$IFDEF SDLMODE}
Procedure SelectColors( M: GearPtr; Redrawer: RedrawProcedureType );
Procedure SelectSprite( M: GearPtr; Redrawer: RedrawProcedureType );
{$ENDIF}

Function FindNextPC( GB: GameBoardPtr; CurrentPC: GearPtr; AllowPets: Boolean ): GearPtr;
Function FindPrevPC( GB: GameBoardPtr; CurrentPC: GearPtr; AllowPets: Boolean ): GearPtr;

Function CanBeExtracted( Item: GearPtr ): Boolean;

Procedure GivePartToPC( GB: GameBoardPtr; Part, PC: GearPtr );

Function SelectRobotParts( GB: GameBoardPtr; PC: GearPtr ): GearPtr;

Procedure DoFieldRepair( GB: GameBoardPtr; PC , Item: GearPtr; Skill: Integer );

Function Handless( Mek: GearPtr ): Boolean;
Function ShakeDown( GB: GameBoardPtr; Part: GearPtr; X,Y: Integer ): LongInt;
{$IFDEF PATCH_GH}
Procedure InstallAmmo( GB: GameBoardPtr; PC , Gun , Ammo: GearPtr );
Function NeedAmmo( Weapon: GearPtr ): Boolean;
Function ReloadAmmo( GB: GameBoardPtr; User,Weapon: GearPtr ): Boolean;
{$ENDIF PATCH_GH}
Procedure PCGetItem( GB: GameBoardPtr; PC: GearPtr );
Procedure StartContinuousUseItem( GB: GameBoardPtr; TruePC , Item: GearPtr );

Procedure FHQ_SelectMechaForPilot( GB: GameBoardPtr; NPC: GearPtr );
Procedure LancemateBackpack( GB: GameBoardPtr; PC,NPC: GearPtr );
Procedure BackpackMenu( GB: GameBoardPtr; PC: GearPtr; StartWithInv: Boolean );
{$IFDEF PATCH_CHEAT}
Procedure RealBackpack( GB: GameBoardPtr; var LList: GearPtr; PC,M: GearPtr; StartWithInv: Boolean );
Procedure MechaPartEditor( GB: GameBoardPtr; var LList: GearPtr; PC,Mek: GearPtr );
{$ENDIF PATCH_CHEAT}

{$IFDEF PATCH_GH}
  {$IFDEF SDLMODE}
Procedure MechaPartBrowser( Mek: GearPtr; DebugMode: Boolean; RDP: RedrawProcedureType );
Procedure MechaPartBrowser( Mek: GearPtr; RDP: RedrawProcedureType );
  {$ELSE}
Procedure MechaPartBrowser( Mek: GearPtr; DebugMode: Boolean );
Procedure MechaPartBrowser( Mek: GearPtr );
  {$ENDIF}
{$ELSE PATCH_GH}
  {$IFDEF SDLMODE}
Procedure MechaPartBrowser( Mek: GearPtr; RDP: RedrawProcedureType );
  {$ELSE}
Procedure MechaPartBrowser( Mek: GearPtr );
  {$ENDIF}
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
Procedure SelectPortrait( GB: GameBoardPtr; M: GearPtr );
Procedure Rename_Mecha( GB: GameBoardPtr; NPC: GearPtr );
Procedure Redesig_Mecha( GB: GameBoardPtr; NPC: GearPtr );
{$IFDEF SDLMODE}
Function SwapMenu_NoParent( var FirstPart: GearPtr; Z: DynamicRect; Part: GearPtr ):Boolean;
{$ELSE SDLMODE}
Function SwapMenu_NoParent( var FirstPart: GearPtr; Z: Integer; Part: GearPtr ):Boolean;
{$ENDIF SDLMODE}
{$IFDEF SDLMODE}
Function SwapMenu( Z: DynamicRect; Part: GearPtr ):Boolean;
{$ELSE SDLMODE}
Function SwapMenu( Z: Integer; Part: GearPtr ):Boolean;
{$ENDIF SDLMODE}
{$ENDIF PATCH_CHEAT}

Procedure FHQ_ThisWargearWasSelected( GB: GameBoardPtr; var LList: GearPtr; PC,M: GearPtr );


implementation

uses
{$IFDEF DEBUG}
	errmsg,
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
	pseudosmartpointer,
{$ENDIF PATCH_GH}
{$IFDEF PATCH_I18N}
	i18nmsg,
{$ENDIF PATCH_I18N}
{$IFDEF PATCH_CHEAT}
	pcaction,
{$ENDIF PATCH_CHEAT}
	ability,action,arenacfe,arenascript,damage,gearutil,ghchars,ghholder,
	ghmodule,ghprop,ghswag,interact,menugear,rpgdice,skilluse,texutil,
{$IFDEF SDLMODE}
	sdlinfo,sdlmap,sdlmenus
	,SDL
{$ELSE}
	congfx,coninfo,conmap,conmenus,context
{$ENDIF}
	,ghweapon,ghintrinsic
{$IFDEF PATCH_GH}
  {$IFDEF SDLMODE}
	,colormenu
  {$ENDIF SDLMODE}
	;
{$ELSE PATCH_GH}
	,colormenu;
{$ENDIF PATCH_GH}

var
	ForceQuit: Boolean;
	EqpRPM,InvRPM: RPGMenuPtr;
{$IFDEF DEBUG}
	EqpRPM_MaxNum, InvRPM_MaxNum: LongInt;
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
	MPB_Gear: GearPtr;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	SP_GB: GameBoardPtr;
	SP_M: GearPtr;
  {$IFDEF SDLMODE}
  {$ELSE SDLMODE}
	InfoGB: GameBoardPtr;
  {$ENDIF SDLMODE}
{$ENDIF PATCH_CHEAT}
{$IFDEF SDLMODE}
	InfoGear: GearPtr;	{ Gear to appear in the INFO menu. }

	BP_Source: GearPtr;	{ Gear whose inventory is being examined. }
	BP_Focus: GearPtr;	{ Gear which is being focused on for FocusOnOneItemRedraw }
	BP_SeekSibs: Boolean;	{ TRUE if the menu lists sibling gears; FALSE if it lists child gears. }
	BP_ActiveMenu: RPGMenuPtr;	{ The active menu. Used to determine the gear to show info about. }
	BPRD_Caption: String;

	InfoGB: GameBoardPtr;
	MPB_Redraw: RedrawProcedureType;
  {$IFDEF PATCH_GH}
	MPR_InvMenu: RPGMenuPtr;
	MPR_InvMenuLGBN: GearPtr;
	MPR_InvMenuRGS: GearPtr;
  {$ENDIF PATCH_GH}

Procedure PlainRedraw;
	{ Miscellaneous menu redraw procedure. }
begin
	if InfoGB <> Nil then SDLCombatDisplay( InfoGB );
	{if InfoGear <> Nil then DisplayGearInfo( InfoGear , InfoGB );}
end;

Procedure EqpRedraw;
	{ Show Inventory, select Equipment. }
var
	N: Integer;
	Part: GearPtr;
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF DEBUG}
	MaxNum := 0;
	if ( NIL <> InfoGB ) then begin
		MaxNum := CountGearIndex( BP_Source, False, '' );
	end;
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
	if ( NIL <> InfoGB ) then begin
		SDLCombatDisplay( InfoGB );
	end;
{$ELSE PATCH_GH}
	SDLCombatDisplay( InfoGB );
{$ENDIF PATCH_GH}
	DrawBPBorder;
	DisplayMenu( InvRPM , Nil );
{$IFDEF PATCH_GH}
	DrawBackpackHeader( InfoGB , BP_Source );
{$ELSE PATCH_GH}
	DrawBackpackHeader( BP_Source );
{$ENDIF PATCH_GH}
	GameMsg( MsgString( 'BACKPACK_Directions' ) , ZONE_BPInstructions.GetRect() , MenuItem );
	if ( BP_ActiveMenu <> Nil ) and ( BP_Source <> Nil ) then begin
		N := CurrentMenuItemValue( BP_ActiveMenu );
		if N > 0 then begin
			if BP_SeekSibs then Part := RetrieveGearSib( BP_Source , N )
{$IFDEF DEBUG}
			else Part := LocateGearByNumber( BP_Source , N, False, MaxNum, 'EqpRedraw' );
{$ELSE DEBUG}
			else Part := LocateGearByNumber( BP_Source , N );
{$ENDIF DEBUG}
			if Part <> Nil then begin
				LongformGearInfo( Part , InfoGB, ZONE_BPInfo );
			end;
		end;
	end;
end;

Procedure InvRedraw;
	{ Show Equipment, select Inventory. }
var
	N: Integer;
	Part: GearPtr;
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF DEBUG}
	MaxNum := 0;
	if ( NIL <> InfoGB ) then begin
		MaxNum := CountGearIndex( BP_Source, False, '' );
	end;
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
	if ( NIL <> InfoGB ) then begin
		SDLCombatDisplay( InfoGB );
	end;
{$ELSE PATCH_GH}
	SDLCombatDisplay( InfoGB );
{$ENDIF PATCH_GH}
	DrawBPBorder;
	DisplayMenu( EqpRPM , Nil );
{$IFDEF PATCH_GH}
	DrawBackpackHeader( InfoGB , BP_Source );
{$ELSE PATCH_GH}
	DrawBackpackHeader( BP_Source );
{$ENDIF PATCH_GH}
	GameMsg( MsgString( 'BACKPACK_Directions' ) , ZONE_BPInstructions.GetRect() , MenuItem );
	if ( BP_ActiveMenu <> Nil ) and ( BP_Source <> Nil ) then begin
		N := CurrentMenuItemValue( BP_ActiveMenu );
		if N > 0 then begin
			if BP_SeekSibs then Part := RetrieveGearSib( BP_Source , N )
{$IFDEF DEBUG}
			else Part := LocateGearByNumber( BP_Source , N, False, MaxNum, 'InvRedraw' );
{$ELSE DEBUG}
			else Part := LocateGearByNumber( BP_Source , N );
{$ENDIF DEBUG}
			if Part <> Nil then begin
				LongformGearInfo( Part , InfoGB, ZONE_BPInfo );
			end;
		end;
	end;
end;

Procedure FocusOnOneItemRedraw;
	{ Miscellaneous menu redraw procedure. The Eqp display will be shown; }
	{ the INV display won't be. }
begin
	if InfoGB <> Nil then SDLCombatDisplay( InfoGB );
	DrawBPBorder;
	LongformGearInfo( BP_Focus , InfoGB, ZONE_BPInfo );
{$IFDEF PATCH_GH}
	DrawBackpackHeader( InfoGB , BP_Source );
{$ELSE PATCH_GH}
	DrawBackpackHeader( BP_Source );
{$ENDIF PATCH_GH}
	if EqpRPM <> Nil then begin
		DisplayMenu( EqpRPM , Nil );
		GameMsg( MsgString( 'BACKPACK_Directions' ) , ZONE_BPInstructions.GetRect() , MenuItem );
	end;
end;

Procedure FHQWargearRedraw;
	{ Do a redraw for the Field HQ. }
begin
	if InfoGB <> Nil then SDLCombatDisplay( InfoGB );
	if BPRD_CAPTION <> '' then begin
		InfoBox( ZONE_FHQTitle.GetRect() );
		CMessage( BPRD_CAPTION , ZONE_FHQTitle.GetRect() , InfoHilight );
	end;
	InfoBox( ZONE_FHQMenu.GetRect() );
	InfoBox( ZONE_FHQInfo.GetRect() );
	LongformGearInfo( InfoGear , InfoGB, ZONE_FHQInfo );
end;

Procedure TransferRedraw;
	{ The redrawer for the transfer frontend. }
var
	Part: GearPtr;
begin
	SDLCombatDisplay( InfoGB );
	if BPRD_CAPTION <> '' then begin
		InfoBox( ZONE_FHQTitle.GetRect() );
		CMessage( BPRD_CAPTION , ZONE_FHQTitle.GetRect() , InfoHilight );
	end;
	InfoBox( ZONE_FHQMenu.GetRect() );
	InfoBox( ZONE_FHQInfo.GetRect() );

	if ( BP_ActiveMenu <> Nil ) and ( BP_Source <> Nil ) then begin
		Part := RetrieveGearSib( BP_Source , CurrentMenuItemValue( BP_ActiveMenu ) );
		if Part <> Nil then begin
			LongformGearInfo( Part , InfoGB, ZONE_FHQInfo );
		end;
	end;
end;


Procedure RobotPartRedraw;
	{ Redraw procedure for the robot part selector. }
begin
	if InfoGB <> Nil then SDLCombatDisplay( InfoGB );
	DrawBPBorder;
	{if InfoGear <> Nil then DisplayGearInfo( InfoGear , InfoGB );}
	GameMsg( MsgString( 'SELECT_ROBOT_PARTS' ) , ZONE_EqpMenu.GetRect() , MenuItem );
end;

Procedure SelectColors( M: GearPtr; Redrawer: RedrawProcedureType );
	{ The player wants to change the colors for this part. Make it so. }
	{ Use the colormenu unit, backported from GH2. }
var
	startpal,mypal: String;
begin
{$IFDEF PATCH_GH}
	if (NIL = M) or (M^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}
	startpal := SAttValue( m^.SA, 'SDL_COLORS' );
	if M^.G = GG_Character then begin
		mypal := SelectColorPalette( colormenu_mode_character, SAttValue( m^.SA, 'SDL_PORTRAIT' ), startpal, 100, 150, 0, Redrawer );
	end else begin
		mypal := SelectColorPalette( colormenu_mode_mecha, SAttValue( m^.SA, 'SDL_SPRITE' ), startpal, 64, 64, 8, Redrawer );
	end;
{$IFDEF PATCH_GH}
	if ( 0 = Length(mypal) ) then Exit;
{$ENDIF PATCH_GH}
	SetSAtt( M^.SA, 'SDL_COLORS <' + mypal + '>' );
end;


Procedure SelectSprite( M: GearPtr; Redrawer: RedrawProcedureType );
	{ The player wants to change the colors for sprite for this character. }
	{ The menu will be placed in the Menu area; assume the redrawer will }
	{ show whatever changes are made here. }
var
	RPM: RPGMenuPtr;
	fname: String;
begin
{$IFDEF PATCH_GH}
	if (NIL = M) or (M^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_CharViewMenu );
{$IFDEF PATCH_CHEAT}
	if Cheat_Unlimited_Sprite then begin
		BuildFileMenu( RPM , Graphics_Directory + 'cha_*.*' );
	end else begin
		if NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) = NAV_Female then begin
			BuildFileMenu( RPM , Graphics_Directory + 'cha_f_*.*' );
		end else if NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) = NAV_Male then begin
			BuildFileMenu( RPM , Graphics_Directory + 'cha_m_*.*' );
		end else begin
			BuildFileMenu( RPM , Graphics_Directory + 'cha_*_*.*' );
		end;
	end;
{$ELSE PATCH_CHEAT}
	if NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) = NAV_Female then begin
		BuildFileMenu( RPM , Graphics_Directory + 'cha_f_*.*' );
	end else if NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) = NAV_Male then begin
		BuildFileMenu( RPM , Graphics_Directory + 'cha_m_*.*' );
	end else begin
		BuildFileMenu( RPM , Graphics_Directory + 'cha_*_*.*' );
	end;
{$ENDIF PATCH_CHEAT}
	AddRPGMenuItem( RPM , MsgString( 'EXIT' ) , -1 );

{$IFDEF PATCH_GH}
	fname := SAttValue( M^.SA , 'SDL_SPRITE' );
	while (RPM^.SelectItem < RPM^.NumItem) do begin
		if RPMLocateByPosition(RPM,RPM^.SelectItem)^.msg = fname then begin
			break;
		end;
		Inc( RPM^.SelectItem );
	end;
	RPM^.TopItem := -1;
{$ENDIF PATCH_GH}

	fname := SelectFile( RPM , Redrawer );

	if fname <> '' then begin
		SetSAtt( M^.SA , 'SDL_SPRITE <' + fname + '>' );
	end;

	DisposeRPGMenu( RPM );
end;

{$ENDIF}



Function FindNextPC( GB: GameBoardPtr; CurrentPC: GearPtr; AllowPets: Boolean ): GearPtr;
	{ Locate the next player character on the gameboard. }
	Function IsPC( PC: GearPtr ): Boolean;
		{ Return True if this is a PC, or False otherwise. }
	var
		team: Longint;
		Pilot: GearPtr;
	begin
		if IsMasterGear( PC ) and GearActive(PC) then begin
			team := NAttValue( PC^.NA , NAG_Location, NAS_Team );
			Pilot := LocatePilot( PC );
			if team = NAV_DefPlayerTeam then IsPC := True
			else if team = NAV_LancemateTeam then IsPC := AllowPets or ( NAttValue( Pilot^.NA , NAG_Personal , NAS_CID ) <> 0 )
			else IsPC := False;
		end else IsPC := False;
	end;
var
	PC,NextPC,FirstPC: GearPtr;
	FoundStart: Boolean;
begin
	NextPC := Nil;
	FirstPC := Nil;
	FoundStart := CurrentPC = Nil;

	PC := GB^.Meks;
	while ( PC <> Nil ) and ( NextPC = Nil ) do begin
		if IsPC(PC) then begin
			if FirstPC = Nil then FirstPC := PC;
			if FoundStart and (NextPC = Nil) then NextPC := PC;
			if PC = CurrentPC then FoundStart := True;
		end;
		PC := PC^.Next;
	end;
	if NextPC = Nil then begin
		if FirstPC = Nil then FindNextPC := CurrentPC
		else FindNextPC := FirstPC;
	end else FindNextPC := NextPC;
end;

Function FindPrevPC( GB: GameBoardPtr; CurrentPC: GearPtr; AllowPets: Boolean ): GearPtr;
	{ Locate the previous player character on the gameboard. }
	Function IsPC( PC: GearPtr ): Boolean;
		{ Return True if this is a PC, or False otherwise. }
	var
		team: Longint;
		Pilot: GearPtr;
	begin
		if IsMasterGear( PC ) and GearActive(PC) then begin
			team := NAttValue( PC^.NA , NAG_Location, NAS_Team );
			Pilot := LocatePilot( PC );
			if team = NAV_DefPlayerTeam then IsPC := True
			else if team = NAV_LancemateTeam then IsPC := AllowPets or ( NAttValue( Pilot^.NA , NAG_Personal , NAS_CID ) <> 0 )
			else IsPC := False;
		end else IsPC := False;
	end;
var
	PC,PrevPC,LastPC: GearPtr;
	FoundStart: Boolean;
begin
	PrevPC := Nil;
	LastPC := Nil;
	FoundStart := CurrentPC = Nil;

	PC := GB^.Meks;
	while ( PC <> Nil ) and not FoundStart do begin
		if IsPC(PC) then begin
			PrevPC := LastPC;
			if PC <> CurrentPC then LastPC := PC
			else if PrevPC <> Nil then FoundStart := True;
		end;
		PC := PC^.Next;
	end;
	if not FoundStart then begin
		if LastPC = Nil then FindPrevPC := CurrentPC
		else FindPrevPC := LastPC;
	end else FindPrevPC := PrevPC;
end;


Function SelectRobotParts( GB: GameBoardPtr; PC: GearPtr ): GearPtr;
	{ Select up to 10 parts to build a robot with. }
	{ Delink them from the INVENTORY and return them as a list. }
var
	Ingredients,Part,P2: GearPtr;
	RPM: RPGMenuPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit(NIL);
{$ENDIF PATCH_GH}

{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
	InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
	DrawBPBorder;
	GameMsg( MsgString( 'SELECT_ROBOT_PARTS' ) , ZONE_EqpMenu , MenuItem );
{$ELSE}
	InfoGB := GB;
	InfoGear := PC;
{$ENDIF}
	Ingredients := Nil;
	repeat
		RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
		RPM^.Mode := RPMNoCleanup;

		Part := PC^.InvCom;
		N := 1;
		while Part <> Nil do begin
{$IFDEF PATCH_GH}
			if (GG_DisposeGear < Part^.G) then begin
				if ( Part^.G = GG_Weapon ) or ( Part^.G = GG_Shield ) or ( Part^.G = GG_ExArmor ) or ( Part^.G = GG_Sensor ) or ( Part^.G = GG_Electronics ) then begin
					AddRPGMenuItem( RPM , GearName( Part ) , N );
				end else if ( Part^.G = GG_RepairFuel ) and ( ( Part^.S = 15 ) or ( Part^.S = 23 ) ) then begin
					AddRPGMenuItem( RPM , GearName( Part ) , N );
				end;
				Inc( N );
			end;
			Part := Part^.Next;
{$ELSE PATCH_GH}
			if ( Part^.G = GG_Weapon ) or ( Part^.G = GG_Shield ) or ( Part^.G = GG_ExArmor ) or ( Part^.G = GG_Sensor ) or ( Part^.G = GG_Electronics ) then begin
				AddRPGMenuItem( RPM , GearName( Part ) , N );
			end else if ( Part^.G = GG_RepairFuel ) and ( ( Part^.S = 15 ) or ( Part^.S = 23 ) ) then begin
				AddRPGMenuItem( RPM , GearName( Part ) , N );
			end;
			Part := Part^.Next;
			Inc( N );
{$ENDIF PATCH_GH}
		end;
		RPMSortAlpha( RPM );
		AlphaKeyMenu( RPM );
		AddRPGMenuItem( RPM , MsgString( 'EXIT' ) , -1 );

{$IFDEF SDLMODE}
		N := SelectMenu( RPM , @RobotPartRedraw );
{$ELSE}
		N := SelectMenu( RPM );
{$ENDIF}
		DisposeRPGMenu( RPM );

		if N > -1 then begin
			Part := RetrieveGearSib( PC^.InvCom , N );
			DelinkGear( PC^.InvCom , Part );
			while Part^.InvCom <> Nil do begin
				P2 := Part^.InvCom;
				DelinkGear( Part^.InvCom , P2 );
				InsertInvCom( PC , P2 );
			end;
			AppendGear( Ingredients , Part );
		end;

	until ( NumSiblingGears( Ingredients ) > 9 ) or ( N = -1 );

	SelectRobotParts := Ingredients;
end;


Procedure AddRepairOptions( RPM: RPGMenuPtr; PC,Item: GearPtr );
	{ Check the object in question, then add options to the }
	{ provided menu if the item is in need of repairs which the }
	{ PC can provide. Repair items will be numbered 100 + RSN }
var
	N: Integer;
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	PC := LocatePilot( PC );
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}
	if PC <> Nil then begin
		for N := 1 to NumRepairSkills do begin
			{ The repair option will only be added to the menu if: }
			{ - The PC has the required skill. }
			{ - The item is in need of repair (using this skill). }
			if ( NAttValue( PC^.NA , NAG_Skill , RepairSkillIndex[N] ) > 0 ) and ( TotalRepairableDamage( Item , RepairSkillIndex[N] ) > 0 ) then begin
{$IFDEF PATCH_I18N}
				AddRPGMenuItem( RPM , ReplaceHash( I18N_MsgString('BACKPACK_Repair'), I18N_Name('SkillMan',SkillMan[ RepairSkillIndex[N] ].Name) ), 100 + N );
{$ELSE PATCH_I18N}
				AddRPGMenuItem( RPM , MsgString( 'BACKPACK_Repair' ) + SkillMan[ RepairSkillIndex[N] ].Name , 100 + N );
{$ENDIF PATCH_I18N}
			end;
		end;
	end;
end;

Procedure DoFieldRepair( GB: GameBoardPtr; PC , Item: GearPtr; Skill: Integer );
	{ The PC is going to use one of the repair skills. Call the }
	{ standard procedure, then print output. }
var
	msg: String;
	N: LongInt;
{$IFDEF PATCH_GH}
	Mek, Pilot: GearPtr;
	MekRepairFuel, PilotRepairFuel, TotalRepairFuel: LongInt;
	RMaster: GearPtr;
{$ELSE PATCH_GH}
	RepairFuel,RMaster: GearPtr;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then begin
		DialogMsg( MsgString( 'PCREPAIR_NoRepairFuel' ) );
		Exit;
	end;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then begin
		DialogMsg( MsgString( 'PCREPAIR_NoDamageDone' ) );
		Exit;
	end;
{$ENDIF PATCH_GH}

	{ Error check - if no repair is needed, display an appropraite }
	{ response. }
	if TotalRepairableDamage( Item , Skill ) < 1 then begin
		DialogMsg( MsgString( 'PCREPAIR_NoDamageDone' ) );
		Exit;
	end;

	{ Locate the "repair fuel". }
{$IFDEF PATCH_GH}
	Mek := FindRoot( PC );
	Pilot := LocatePilot( PC );
	MekRepairFuel := 0;
	PilotRepairFuel := 0;
	if ( ( NIL <> Mek ) and ( Mek <> Pilot ) ) then begin
		MekRepairFuel := CheckAndDeplete_RepairFuel( Mek , Skill , 0 , '' , msg );
	end;
	if ( NIL <> Pilot ) then begin
		PilotRepairFuel := CheckAndDeplete_RepairFuel( Pilot , Skill , 0 , '' , msg );
	end;
	TotalRepairFuel := MekRepairFuel + PilotRepairFuel;
	if ( TotalRepairFuel <= 0 ) then begin
		DialogMsg( MsgString( 'PCREPAIR_NoRepairFuel' ) );
		Exit;
	end;
{$ELSE PATCH_GH}
	RepairFuel := SeekGear( PC , GG_RepairFuel , Skill );
	if RepairFuel = Nil then begin
		DialogMsg( MsgString( 'PCREPAIR_NoRepairFuel' ) );
		Exit;
	end;
{$ENDIF PATCH_GH}

	{ Locate the root item. If this is a character, and the repair attempt }
	{ fails, and the master is destroyed, that's a bad thing. }
	RMaster := FindRoot( Item );

{$IFDEF PATCH_GH}
	N := UseRepairSkill( GB , PC , Item , Skill , TotalRepairFuel );
{$ELSE PATCH_GH}
	N := UseRepairSkill( GB , PC , Item , Skill );
{$ENDIF PATCH_GH}
	msg := ReplaceHash( MsgString( 'PCREPAIR_UseSkill' + BStr( Skill ) ) , GearName( Item ) );

	{ Inform the user of the success. }
	if ( RMaster^.G = GG_Character ) and Destroyed( RMaster ) then begin
		AddNAtt( RMaster^.NA , NAG_Damage , NAS_StrucDamage , 30 );
		msg := msg + ReplaceHash( MsgString( 'PCREPAIR_DEAD' ) , GearName( RMaster ) );
	end else if N > 0 then begin
		msg := msg + BStr( N ) + MsgString( 'PCREPAIR_Success' + BStr( Skill ) );
	end else begin
		msg := msg + MsgString( 'PCREPAIR_Failure' + BStr( Skill ) );
	end;

	DialogMsg( msg );

	{ Deplete the fuel. }
{$IFDEF PATCH_GH}
	if ( ( NIL <> Mek ) and ( Mek <> Pilot ) ) then begin
		msg := '';
		CheckAndDeplete_RepairFuel( Mek , Skill , N , MsgString( 'PCREPAIR_FuelUsedUp' ) , msg );
		if ( '' <> msg ) then begin
			DialogMsg( msg );
		end;
	end;
	if ( ( MekRepairFuel < N ) and ( NIL <> Pilot ) ) then begin
		msg := '';
		CheckAndDeplete_RepairFuel( Pilot , Skill , ( N - MekRepairFuel ) , MsgString( 'PCREPAIR_FuelUsedUp' ) , msg );
		if ( '' <> msg ) then begin
			DialogMsg( msg );
		end;
	end;
{$ELSE PATCH_GH}
	RepairFuel^.V := RepairFuel^.V - N;
	if RepairFuel^.V < 1 then begin
		DialogMsg( ReplaceHash( MsgString( 'PCREPAIR_FuelUsedUp' ) , GearName( RepairFuel ) ) );
		if IsSubCom( RepairFuel ) then begin
			RemoveGear( RepairFuel^.Parent^.SubCom , RepairFuel );
		end else if IsInvCom( RepairFuel ) then begin
			RemoveGear( RepairFuel^.Parent^.InvCom , RepairFuel );
		end;
	end;
{$ENDIF PATCH_GH}
end;

Function ShakeDown( GB: GameBoardPtr; Part: GearPtr; X,Y: Integer ): LongInt;
	{ This is the workhorse for this function. It does the }
	{ dirty work of separating inventory from (former) owner. }
const
	V_MAX = 2147483647;
	V_MIN = -2147483648;

{$IFDEF PATCH_GH}
	Function ShakeDown_internal( GB: GameBoardPtr; Part: GearPtr; X,Y: Integer ): Int64;
	var
		Cash: Int64;
		SPart: GearPtr;		{ Sub-Part }
	begin
		{ Start by removing the cash from this part. }
		Cash := NAttValue( Part^.NA , NAG_Experience , NAS_Credits );
		SetNAtt( Part^.NA , NAG_Experience , NAS_Credits , 0 );
		SetNAtt( Part^.NA , NAG_EpisodeData , NAS_Ransacked , 1 );

		{ Remove all InvComs, and place them on the map. }
		While (NIL <> Part^.InvCom) do begin
			SPart := Part^.InvCom;
			DelinkGear( Part^.InvCom , SPart );
			{ If this invcom isn't destroyed, put it on the }
			{ ground for the PC to pick up. Otherwise delete it. }
			if NotDestroyed( SPart ) then begin
				SetNAtt( SPart^.NA , NAG_Location , NAS_X , X );
				SetNAtt( SPart^.NA , NAG_Location , NAS_Y , Y );
				SPart^.Next := GB^.Meks;
				GB^.Meks := SPart;
			end else begin
				DisposeGear( SPart );
			end;
		end;

		{ Shake down this gear's subcoms. }
		SPart := Part^.SubCOm;
		while (NIL <> SPart) do begin
			if (GG_DisposeGear < SPart^.G) then begin
				if SPart^.G <> GG_Cockpit then begin
					Cash := Cash + ShakeDown_internal( GB , SPart , X , Y );
				end;
			end;
			SPart := SPart^.Next;
		end;

		if (V_MAX < Cash) then begin
			Cash := V_MAX;
		end else if (Cash < V_MIN) then begin
			Cash := V_MIN;
		end;
		ShakeDown_internal := Cash;
	end;
begin
	if (NIL = Part) or (Part^.G <= GG_DisposeGear) then Exit(0);
	ShakeDown := ShakeDown_internal( GB, Part, X, Y );
{$ELSE PATCH_GH}
var
	Cash: Int64;
	SPart: GearPtr;		{ Sub-Part }
begin
	{ Start by removing the cash from this part. }
	cash := NAttValue( Part^.NA , NAG_Experience , NAS_Credits );
	SetNAtt( Part^.NA , NAG_Experience , NAS_Credits , 0 );
	SetNAtt( Part^.NA , NAG_EpisodeData , NAS_Ransacked , 1 );

	{ Remove all InvComs, and place them on the map. }
	While Part^.InvCom <> Nil do begin
		SPart := Part^.InvCom;
		DelinkGear( Part^.InvCom , SPart );
		{ If this invcom isn't destroyed, put it on the }
		{ ground for the PC to pick up. Otherwise delete it. }
		if NotDestroyed( SPart ) then begin
			SetNAtt( SPart^.NA , NAG_Location , NAS_X , X );
			SetNAtt( SPart^.NA , NAG_Location , NAS_Y , Y );
			SPart^.Next := GB^.Meks;
			GB^.Meks := SPart;
		end else begin
			DisposeGear( SPart );
		end;
	end;

	{ Shake down this gear's subcoms. }
	SPart := Part^.SubCOm;
	while SPart <> Nil do begin
		if SPart^.G <> GG_Cockpit then cash := cash + ShakeDown( GB , SPart , X , Y );
		SPart := SPart^.Next;
	end;

	if (V_MAX < Cash) then begin
		Cash := V_MAX;
	end else if (Cash < V_MIN) then begin
		Cash := V_MIN;
	end;

	ShakeDown := Cash;
{$ENDIF PATCH_GH}
end;


Function Ransack( GB: GameBoardPtr; X,Y: Integer ): LongInt;
	{ Yay! Loot and pillage! This function has two purposes: }
	{ first, it separates all Inventory gears from any non-operational }
	{ masters standing in this tile. Secondly, it collects the }
	{ money from all those non-operational masters and returns the }
	{ total amount as the function result. }
var
	it: LongInt;
	Mek: GearPtr;
begin
	it := 0;

	Mek := GB^.Meks;

	while Mek <> Nil do begin
{$IFDEF PATCH_GH}
		if (GG_DisposeGear < Mek^.G) then begin
{$ENDIF PATCH_GH}
		{ If this is a broken-down master, check to see if it's }
		{ one we want to pillage. }
		if IsMasterGear( Mek ) and not GearOperational( Mek ) then begin
			{ We will ransack this gear if it's in the correct location. }
			if ( NAttValue( Mek^.NA , NAG_Location , NAS_X ) = X ) and ( NAttValue( Mek^.NA , NAG_Location , NAS_Y ) = Y ) then begin
				it := it + ShakeDown( GB , Mek , X , Y );
			end;
		end else if ( Mek^.G = GG_MetaTerrain ) and ( ( Mek^.Stat[ STAT_Lock ] = 0 ) or Destroyed( Mek ) ) then begin
			{ Metaterrain gets ransacked if it's unlocked, }
			{ or wrecked. }
			if ( NAttValue( Mek^.NA , NAG_Location , NAS_X ) = X ) and ( NAttValue( Mek^.NA , NAG_Location , NAS_Y ) = Y ) then begin
				it := it + ShakeDown( GB , Mek , X , Y );
			end;
		end;
{$IFDEF PATCH_GH}
		end;
{$ENDIF PATCH_GH}
		Mek := Mek^.Next;
	end;

	Ransack := it;
end;

Function Handless( Mek: GearPtr ): Boolean;
	{ Return TRUE if Mek either has no hands or can't use its hands }
	{ at the moment (say, because it's transformed into tank mode). }
	{ Return TRUE if Mek has hands and they are in perfect working order. }
var
	Hand: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit(True);
  {$IFDEF PATCH_CHEAT}
	Hand := SeekActiveIntrinsic( Mek , GG_Holder , GS_Hand , Cheat_EnableCockpitBarrier_Hand );
  {$ELSE PATCH_CHEAT}
	Hand := SeekActiveIntrinsic( Mek , GG_Holder , GS_Hand );
  {$ENDIF PATCH_CHEAT}
	if (NIL = Hand) or (Hand^.G <= GG_DisposeGear) then Exit(True);
	Handless := not InGoodModule( Hand );
{$ELSE PATCH_GH}
	Hand := SeekActiveIntrinsic( Mek , GG_Holder , GS_Hand );
	if Hand = Nil then Handless := True
	else Handless := not InGoodModule( Hand );
{$ENDIF PATCH_GH}
end;

{$IFDEF SDLMODE}
	Procedure GetItemRedraw;
	begin
  {$IFDEF PATCH_GH}
		if (NIL <> InfoGB) then begin
			SDLCombatDisplay( InfoGB );
		end;
		InfoBox( ZONE_CenterMenu.GetRect() );
		if (NIL <> InfoGear) and (GG_DisposeGear < InfoGear^.G) then begin
			{DisplayGearInfo( InfoGear , InfoGB );}
		end;
  {$ELSE PATCH_GH}
		SDLCombatDisplay( InfoGB );
		InfoBox( ZONE_CenterMenu.GetRect() );
		{DisplayGearInfo( InfoGear , InfoGB );}
  {$ENDIF PATCH_GH}
	end;
{$ENDIF}

{$IFDEF PATCH_GH}
Function SelectVisibleItem( GB: GameBoardPtr; PC: GearPtr; X,Y: Integer; var FlagAll: Boolean ): GearPtr;
{$ELSE PATCH_GH}
Function SelectVisibleItem( GB: GameBoardPtr; PC: GearPtr; X,Y: Integer ): GearPtr;
{$ENDIF PATCH_GH}
	{ Attempt to select a visible item from gameboard tile X,Y. }
	{ If more than one item is present, prompt the user for which one }
	{ to pick up. }
var
	N,T: Integer;
	RPM: RPGMenuPtr;
begin
{$IFDEF PATCH_GH}
	FlagAll := False;
{$ENDIF PATCH_GH}

	{ First count the number of items in this spot. }
	N := NumVisibleItemsAtSpot( GB , X , Y );

	{ If it's just 0 or 1, then our job is simple... }
	if N = 0 then begin
		SelectVisibleItem := Nil;
	end else if N = 1 then begin
		SelectVisibleItem := FindVisibleItemAtSpot( GB , X , Y );

	{ If it's more than one, better create a menu and let the user }
	{ pick one. }
	end else if N > 1 then begin
		DialogMsg( MsgString( 'GET_WHICH_ITEM?' ) );
{$IFDEF SDLMODE}
		RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_CenterMenu );
{$ELSE}
		RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_Menu );
{$ENDIF}
		for t := 1 to N do begin
			AddRPGMenuItem( RPM , GearName( GetVisibleItemAtSpot( GB , X , Y , T ) ) , T );
		end;
{$IFDEF PATCH_GH}
		if ( 1 < RPM^.NumItem ) then begin
			AddRPGMenuItem_Top( RPM , 0 , I18N_MsgString( 'SelectVisibleItem', 'PickUpAll' ) , -5 , '' );
		end;
{$ENDIF PATCH_GH}
{$IFDEF SDLMODE}
  {$IFDEF PATCH_GH}
		if (NIL = PC) or (PC^.G <= GG_DisposeGear) then InfoGear := NIL else InfoGear := PC;
  {$ELSE PATCH_GH}
		InfoGear := PC;
  {$ENDIF PATCH_GH}
		InfoGB := GB;
		N := SelectMenu( RPM , @GetItemRedraw );
{$ELSE}
  {$IFDEF PATCH_CHEAT}
		InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
		N := SelectMenu( RPM );
{$ENDIF}
		DisposeRPGMenu( RPM );
		if N > -1 then begin
			SelectVisibleItem := GetVisibleItemAtSpot( GB , X , Y , N );
{$IFDEF PATCH_GH}
		end else if ( -5 = N ) then begin
			FlagAll := True;
			SelectVisibleItem := NIL;
{$ENDIF PATCH_GH}
		end else begin
			SelectVisibleItem := Nil;
		end;
	end;
end;

Procedure PCGetItem( GB: GameBoardPtr; PC: GearPtr );
	{ The PC will attempt to pick up something lying on the ground. }

{$IFDEF PATCH_GH}
var
	Cash: LongInt;

	Procedure GetItem( Item: GearPtr );
	var
		NID: LongInt;
	begin
		if ( NIL <> Item ) then begin
			if IsLegalSlot( PC , Item ) then begin
				DelinkGear( GB^.Meks , Item );

				{ Clear the item's location values. }
				StripNAtt( Item , NAG_Location );
				if NAttValue( Item^.NA, NAG_ParaLocation, NAS_OriginalHome ) < 0 then begin
					StripNAtt( Item, NAG_ParaLocation );
				end;

				InsertInvCom( PC , Item );
				{ Clear the home, to prevent wandering items. }
				SetSAtt( Item^.SA , 'HOME <>' );
				DialogMsg( ReplaceHash( MsgString( 'YOU_GET_?' ) , GearName( Item ) ) );

				NID := NAttValue( Item^.NA , NAG_Narrative , NAS_NID );
				if NID <> 0 then SetTrigger( GB , TRIGGER_GetItem + BStr( NID ) );
			end else if Cash = 0 then begin
				DialogMsg( ReplaceHash( MsgString( 'CANT_GET_?' ) , GearName( Item ) ) );
			end;
		end else if Cash = 0 then begin
  {$IFDEF PATCH_I18N}
			DialogMSG( I18N_MsgString('PCGetItem','No item found') );
  {$ELSE PATCH_I18N}
			DialogMSG( 'No item found.' );
  {$ENDIF PATCH_I18N}
		end;

		if Cash > 0 then begin
			DialogMsg( ReplaceHash( MsgString( 'YouFind$' ) , BStr( Cash ) ) );
			AddNAtt( LocatePilot( PC )^.NA , NAG_Experience , NAS_Credits , Cash );
			Cash := 0;
		end;

		{ Picking up an item takes time. }
		WaitAMinute( GB , PC , ReactionTime( PC ) );
	end;
	Procedure GetAllItem( P: Point );
	var
		Match: LPattern;
		M, M_Next: GearPtr;
	begin
		Match.X := P.X;
		Match.Y := P.Y;
		Match.Z := -10;
		Match.Only_Visibles := True;
		Match.Only_Masters := LP_MustNotBeMaster;

		M := GB^.Meks;
		while ( NIL <> M ) do begin
			M_Next := M^.Next;
			if GearMatchesLPattern( GB , M , Match ) then begin
				GetItem( M );
			end;
			M := M_Next;
		end;
	end;
var
	P: Point;
	item: GearPtr;
	FlagAll: Boolean;
{$ELSE PATCH_GH}
var
	Cash,NID: LongInt;
	P: Point;
	item: GearPtr;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) or Handless(PC) then begin
{$ELSE PATCH_GH}
	if Handless( PC ) then begin
{$ENDIF PATCH_GH}
		{ Start by checking something that other RPGs would }
		{ just assume- does the PC have any hands? }
{$IFDEF PATCH_I18N}
		DialogMsg( I18N_MsgString('PCGetItem','you need hands') );
{$ELSE PATCH_I18N}
		DialogMsg( 'You need hands in order to use this command.' );
{$ENDIF PATCH_I18N}

	end else begin
		P := GearCurrentLocation( PC );

		{ Before attempting to get an item, ransack whatever }
		{ fallen enemies lie in this spot. }
		Cash := Ransack( GB , P.X , P.Y );

		{ Perform an immediate vision check- without it, items }
		{ freed by the Ransack procedure above will remain unseen. }
		VisionCheck( GB , PC );

{$IFDEF PATCH_GH}
		Item := SelectVisibleItem( GB , PC , P.X , P.Y , FlagAll );

		if FlagAll then begin
			GetAllItem( P );
		end else begin
			GetItem( Item );
		end;
{$ELSE PATCH_GH}
		Item := SelectVisibleItem( GB , PC , P.X , P.Y );

		if Item <> Nil then begin
			if IsLegalSlot( PC , Item ) then begin
				DelinkGear( GB^.Meks , Item );

				{ Clear the item's location values. }
				StripNAtt( Item , NAG_Location );
{$IFDEF PATCH_GH}
				if NAttValue( Item^.NA, NAG_ParaLocation, NAS_OriginalHome ) < 0 then begin
					StripNAtt( Item, NAG_ParaLocation );
				end;
{$ENDIF PATCH_GH}

				InsertInvCom( PC , Item );
				{ Clear the home, to prevent wandering items. }
				SetSAtt( Item^.SA , 'HOME <>' );
				DialogMsg( ReplaceHash( MsgString( 'YOU_GET_?' ) , GearName( Item ) ) );

				NID := NAttValue( Item^.NA , NAG_Narrative , NAS_NID );
				if NID <> 0 then SetTrigger( GB , TRIGGER_GetItem + BStr( NID ) );
			end else if Cash = 0 then begin
				DialogMsg( ReplaceHash( MsgString( 'CANT_GET_?' ) , GearName( Item ) ) );
			end;
		end else if Cash = 0 then begin
{$IFDEF PATCH_I18N}
			DialogMSG( I18N_MsgString('PCGetItem','No item found') );
{$ELSE PATCH_I18N}
			DialogMSG( 'No item found.' );
{$ENDIF PATCH_I18N}
		end;

		if Cash > 0 then begin
			DialogMsg( ReplaceHash( MsgString( 'YouFind$' ) , BStr( Cash ) ) );
			AddNAtt( LocatePilot( PC )^.NA , NAG_Experience , NAS_Credits , Cash );
		end;

		{ Picking up an item takes time. }
		WaitAMinute( GB , PC , ReactionTime( PC ) );
{$ENDIF PATCH_GH}
	end;
end;

Procedure CreateInvMenu( PC: GearPtr );
	{ Allocate the Inventory menu and fill it up with the PC's inventory. }
begin
	if InvRPM <> Nil then DisposeRPGMenu( InvRPM );
	InvRPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
	InvRPM^.Mode := RPMNoCleanup;
{$IFDEF PATCH_CHEAT}
  {$IFDEF DEBUG}
	InvRPM_MaxNum := BuildInventoryMenu( InvRPM , PC , Cheat_InvMenu_ShowSubItem );
  {$ELSE DEBUG}
	BuildInventoryMenu( InvRPM , PC , Cheat_InvMenu_ShowSubItem );
  {$ENDIF DEBUG}
{$ELSE PATCH_CHEAT}
  {$IFDEF DEBUG}
	InvRPM_MaxNum := BuildInventoryMenu( InvRPM , PC );
  {$ELSE DEBUG}
	BuildInventoryMenu( InvRPM , PC );
  {$ENDIF DEBUG}
{$ENDIF PATCH_CHEAT}
{$IFNDEF SDLMODE}
	AttachMenuDesc( InvRPM , ZONE_Menu2 );
{$ENDIF}
{$IFDEF PATCH_CHEAT}
	if not(Cheat_InvMenu_NoSort) then begin
		if Cheat_InvMenu_ShowSubItem then begin
			RPMSortAlpha_withSubItem( InvRPM );
		end else begin
			RPMSortAlpha( InvRPM );
		end;
	end;
{$ELSE PATCH_CHEAT}
	RPMSortAlpha( InvRPM );
{$ENDIF PATCH_CHEAT}

	{ If the menu is empty, add a message saying so. }
{$IFDEF PATCH_I18N}
	If InvRPM^.NumItem < 1 then AddRPGMenuItem( InvRPM , I18N_MsgString('CreateInvMenu','no inventory items') , -1 )
{$ELSE PATCH_I18N}
	If InvRPM^.NumItem < 1 then AddRPGMenuItem( InvRPM , '[no inventory items]' , -1 )
{$ENDIF PATCH_I18N}
	else AlphaKeyMenu( InvRPM );

	{ Add the menu keys. }
	AddRPGMenuKey(InvRPM,'/',-2);
{$IFDEF SDLMODE}
	AddRPGMenuKey( InvRPM , RPK_Right ,  -3 );
	AddRPGMenuKey( InvRPM , RPK_Left , -4 );
  {$IFDEF PATCH_GH}
	AddRPGMenuKey( InvRPM , KeyMap[ KMC_Right ].KCode ,  -3 );
	AddRPGMenuKey( InvRPM , KeyMap[ KMC_Left ].KCode , -4 );
  {$ENDIF PATCH_GH}
{$ELSE}
	AddRPGMenuKey( InvRPM , KeyMap[ KMC_East ].KCode , -3 );
	AddRPGMenuKey( InvRPM , KeyMap[ KMC_West ].KCode , -4 );
{$ENDIF}
{$IFDEF PATCH_CHEAT}
	if Cheat_MenuOrder_Edit then begin
		AddRPGMenuKey( InvRPM , KeyMap[ KMC_EditMenuOrder ].KCode , -128 );
	end;
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_GH}
	if ( 1 < InvRPM^.NumItem ) then begin
		AddRPGMenuItem_Top( InvRPM , 0 , I18N_MsgString( 'ThisItemWasSelected', 'DropAll' ), -5 , '' );
		AddRPGMenuItem_Top( InvRPM , 0 , I18N_MsgString( 'ThisItemWasSelected', 'TransferAll' ), -6 , '' );
	end;
{$ENDIF PATCH_GH}
end;

Procedure CreateEqpMenu( PC: GearPtr );
	{ Allocate the equipment menu and fill it up with the PC's gear. }
{$IFDEF PATCH_CHEAT}
	Function AddRPGMenu_MassMeter( RPM: RPGMenuPtr; PC: GearPtr): String;
		{ On top of the backpack seperator, display the current inventory mass, and the carrying capacity }
		{ Origin : Michael }
		{ http://gearheadrpg.com/forum/index.php?action=vthread&forum=1&topic=789 }
		{ ftp://ftp.ocis.net/pub/users/ldeutsch/ghpatches/gh-1100-massmeter1.diff }
	var
		MassString: String;
		CurrentInv,CurrentEqp,Limit,Limit2: LongInt;
	begin
		CurrentInv := IntrinsicMass(PC);
		CurrentEqp := EquipmentMass(PC);
		Limit2 := GearEncumberance(PC);

		Limit := Limit2 * 2 - 1;	{Maximum weight before penalty starts}

		if PC^.G = GG_Character then begin
			Limit := Limit + NAttValue(PC^.NA,NAG_Skill,NAS_WeightLifting);
		end;

		Limit2 := Limit2 + Limit;	{Where penalty gets worse}

		MassString := ReplaceHash( I18N_MsgString('CreateEqpMenu','MassMeter'),
					MakeMassString(CurrentInv,PC^.Scale),
					MakeMassString(CurrentEqp,PC^.Scale) );
		AddRPGMenuItem_Top( RPM, 0 , #$0 + MassString, 0 , '' );

		MassString := ReplaceHash( I18N_MsgString('CreateEqpMenu','MassMeter_Limit'),
					MakeMassString(Limit,PC^.Scale) );
		if Limit2 < CurrentEqp then begin
			MassString := MassString + I18N_MsgString('CreateEqpMenu','MassMeter_LimitOver2');
		end else if Limit < CurrentEqp then begin
			MassString := MassString + I18N_MsgString('CreateEqpMenu','MassMeter_LimitOver');
		end else begin
		end;
		AddRPGMenuItem_Top( RPM, 0 , #$0 + MassString, 0 , '' );
	end;
{$ENDIF PATCH_CHEAT}
begin
	if EqpRPM <> Nil then DisposeRPGMenu( EqpRPM );
	EqpRPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_EqpMenu );
	EqpRPM^.Mode := RPMNoCleanup;
{$IFNDEF SDLMODE}
	AttachMenuDesc( EqpRPM , ZONE_Menu2 );
{$ENDIF}
{$IFDEF DEBUG}
	EqpRPM_MaxNum := BuildEquipmentMenu( EqpRPM , PC );
{$ELSE DEBUG}
	BuildEquipmentMenu( EqpRPM , PC );
{$ENDIF DEBUG}

	{ If the menu is empty, add a message saying so. }
{$IFDEF PATCH_I18N}
	If EqpRPM^.NumItem < 1 then AddRPGMenuItem( EqpRPM , #$0 + I18N_MsgString('CreateEqpMenu','no equipped items') , -1 );
{$ELSE PATCH_I18N}
	If EqpRPM^.NumItem < 1 then AddRPGMenuItem( EqpRPM , '[no equipped items]' , -1 );
{$ENDIF PATCH_I18N}

	{ Add the menu keys. }
	AddRPGMenuKey(EqpRPM,'/',-2);

{$IFDEF PATCH_CHEAT}
	if Cheat_EqpMenu_AddMenuKey then begin
		AlphaKeyMenu( EqpRPM );
	end;
{$ENDIF PATCH_CHEAT}
{$IFDEF SDLMODE}
	AddRPGMenuKey( EqpRPM , RPK_Right ,  -3 );
	AddRPGMenuKey( EqpRPM , RPK_Left , -4 );
  {$IFDEF PATCH_GH}
	AddRPGMenuKey( InvRPM , KeyMap[ KMC_Right ].KCode ,  -3 );
	AddRPGMenuKey( InvRPM , KeyMap[ KMC_Left ].KCode , -4 );
  {$ENDIF PATCH_GH}
{$ELSE}
	AddRPGMenuKey( EqpRPM , KeyMap[ KMC_East ].KCode , -3 );
	AddRPGMenuKey( EqpRPM , KeyMap[ KMC_West ].KCode , -4 );
{$ENDIF}
{$IFDEF PATCH_GH}
	if ( 1 < EqpRPM^.NumItem ) then begin
		AddRPGMenuItem_Top( EqpRPM , 0 , I18N_MsgString( 'ThisItemWasSelected', 'DropAll' ), -5 , '' );
		AddRPGMenuItem_Top( EqpRPM , 0 , I18N_MsgString( 'ThisItemWasSelected', 'TransferAll' ), -6 , '' );
	end;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	If Cheat_EqpMenu_ShowMassMeter then begin
		if (NIL <> PC) and (GG_DisposeGear < PC^.G) then begin
			AddRPGMenu_MassMeter( EqpRPM, PC );
		end;
	end;
{$ENDIF PATCH_CHEAT}
end;

Procedure UpdateBackpack( PC: GearPtr );
	{ Redo all the menus, and display them on the screen. }
begin
	CreateInvMenu( PC );
	CreateEqpMenu( PC );
{$IFNDEF SDLMODE}
	DisplayMenu( InvRPM );
	DisplayMenu( EqpRPM );
{$ENDIF}
end;

Procedure GivePartToPC( GB: GameBoardPtr; Part, PC: GearPtr );
	{ Give the specified part to the PC. If the part cannot be }
	{ held by the PC, store it so that it can be recovered using }
	{ the FieldHQ Wargear Explorer. }
var
	team: Integer;
begin
{$IFDEF PATCH_GH}
	if (NIL = Part) or (Part^.G <= GG_DisposeGear) then Exit;
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = GB) then begin
		InsertInvCom( PC, Part );
	end else
{$ENDIF PATCH_GH}
	if ( PC <> Nil ) and IsLegalSlot( PC , Part ) then begin
		InsertInvCom( PC , Part );
	end else begin
		{ If the PC can't carry this equipment, }
		{ stick it off the map. }
		team := NattValue( PC^.NA , NAG_Location , NAS_Team );
		if team = NAV_LancemateTeam then team := NAV_DefPlayerTeam;
		SetNAtt( Part^.NA , NAG_Location , NAS_Team , team );
		DeployMek( GB , Part , False );
	end;
end;

Procedure UnequipItem( GB: GameBoardPtr; PC , Item: GearPtr );
	{ Delink ITEM from its parent, and stick it in the general inventory... }
	{ If possible. Otherwise drop it. }
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}
	{ First, delink Item from its parent. }
	DelinkGear( Item^.Parent^.InvCom , Item );
	{ HOW'D YA LIKE THEM CARROT DOTS, EH!?!? }

	{ Next, link ITEM into the general inventory. }
	GivePartToPC( GB , Item , PC );

	{ Unequipping takes time. }
	if GB <> Nil then WaitAMinute( GB , PC , ReactionTime( PC ) );
end;

Procedure UnequipFrontend( GB: GameBoardPtr; PC , Item: GearPtr );
	{ Simply unequip the provided item. }
	{ PRECOND: PC and ITEM had better be correct, dagnabbit... }
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_I18N}
	DialogMsg( ReplaceHash( I18N_MsgString('UnequipFrontend','You unequip'), GearName(Item) ) );
{$ELSE PATCH_I18N}
	DialogMsg( 'You unequip ' + GearName( Item ) + '.' );
{$ENDIF PATCH_I18N}
	UnequipItem( GB , PC , Item );
end;


Function CanBeExtracted( Item: GearPtr ): Boolean;
	{ Return TRUE if the listed part can be extracted from a mecha, }
	{ or FALSE if it cannot normally be extracted. }
begin
{$IFDEF PATCH_GH}
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	if ( Item^.G = GG_Support ) then begin
		CanBeExtracted := False;
		if Cheat_MechaCustomize_FreeSupport then begin
			CanBeExtracted := True;
		end;
	end else if ( Item^.G = GG_Cockpit ) then begin
		CanBeExtracted := False;
		if Cheat_MechaCustomize_FreeCockpit then begin
			CanBeExtracted := True;
		end;
	end else if IsMasterGear( Item ) then begin
		CanBeExtracted := False;
		if Cheat_MechaCustomize_FreeMasterGear then begin
			CanBeExtracted := True;
		end;
	end else if ( Item^.Parent = Nil ) then begin
		CanBeExtracted := False;
		if Cheat_MechaCustomize_FreeParent then begin
			CanBeExtracted := True;
		end;
	end else if ( Item^.Parent^.Scale = 0 ) then begin
		CanBeExtracted := False;
		if Cheat_MechaCustomize_FreeScale then begin
			CanBeExtracted := True;
		end;
	end else if ( Item^.G = GG_Modifier ) then begin
		CanBeExtracted := False;
		if Cheat_MechaCustomize_FreeModifier then begin
			CanBeExtracted := True;
		end;
	end else if ( Item^.G = GG_Module ) and ( Item^.S = GS_Body ) then begin
		CanBeExtracted := False;
		if Cheat_MechaCustomize_FreeBodyModule then begin
			CanBeExtracted := True;
		end;
	end else if SeekGear( Item , GG_Cockpit , 0 , False ) <> Nil then begin
		{ If the item contains the cockpit, it can't be extracted. }
		CanBeExtracted := False;
	end else begin
		CanBeExtracted := Not PartHasIntrinsic( Item , NAS_Integral );
	end;
	if Cheat_MechaCustomize_Limitless then begin
		CanBeExtracted := True;
	end;
{$ELSE PATCH_CHEAT}
	if ( Item^.G = GG_Support ) or ( Item^.G = GG_Cockpit ) or IsMasterGear( Item ) or ( Item^.Parent = Nil ) or ( Item^.Parent^.Scale = 0 ) or ( Item^.G = GG_Modifier ) then begin
		CanBeExtracted := False;
	end else if ( Item^.G = GG_Module ) and ( Item^.S = GS_Body ) then begin
		CanBeExtracted := False;
	end else if SeekGear( Item , GG_Cockpit , 0 , False ) <> Nil then begin
		{ If the item contains the cockpit, it can't be extracted. }
		CanBeExtracted := False;
	end else begin
		CanBeExtracted := Not PartHasIntrinsic( Item , NAS_Integral );
	end;
{$ENDIF PATCH_CHEAT}
end;

{$IFDEF PATCH_CHEAT}
Function ExtractItem( GB: GameBoardPtr; LList, TruePC, PC: GearPtr; var Item: GearPtr ): Boolean;
{$ELSE PATCH_CHEAT}
Function ExtractItem( GB: GameBoardPtr; TruePC , PC: GearPtr; var Item: GearPtr ): Boolean;
{$ENDIF PATCH_CHEAT}
	{ Delink ITEM from its parent, and stick it in the general inventory. }
	{ Note that pulling a gear out of its mecha may well wreck it }
	{ beyond any repair! Therefore, after this call, ITEM might no }
	{ longer exist... i.e. it may equal NIL. }
var
	it: Boolean;
	SkTarget,SkRoll,WreckTarget: Integer;
{$IFDEF PATCH_CHEAT}
	MaxHP_Before: LongInt;
	msg: String;
{$ENDIF PATCH_CHEAT}
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the NIL of GB at here. }
	{ Don't kick out the GG_DisposeGear of LList at here. }
	if (NIL = TruePC) or (TruePC^.G <= GG_DisposeGear) then Exit(False);
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit(False);
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}

	{ First, calculate the skill target. }
	SkTarget := 2 + ComponentComplexity( Item );
	if Item^.G = GG_Module then begin
		WreckTarget := SkTarget + 8 - Item^.V;
	end else if Item^.Scale < Item^.Parent^.Scale then begin
		WreckTarget := SkTarget + 5;
	end else begin
		WreckTarget := SkTarget + 10 - UnscaledMaxDamage( Item );
	end;
	if WreckTarget < SkTarget then WreckTarget := SkTarget + 1;

{$IFDEF PATCH_GH}
	if (NIL <> GB) then begin
		SkRoll := RollStep( TeamSkill( GB , NAV_DefPlayerTeam , 31 ) );
	end else if (NIL <> LList) then begin
		SkRoll := RollStep( TeamSkill( LList, 31 ) );
	end else begin
		SkRoll := RollStep( SkillValue( TruePC, 31 ) );
	end;
{$ELSE PATCH_GH}
	SkRoll := RollStep( TeamSkill( GB , NAV_DefPlayerTeam , 31 ) );
{$ENDIF PATCH_GH}

	DoleSkillExperience( TruePC , 31 , 1 );
{$IFDEF PATCH_CHEAT}
	if GearOperational( TruePC ) then begin
		MaxHP_Before := GearMaxDamage( TruePC );
	end else begin
		MaxHP_Before := 0;
	end;
{$ENDIF PATCH_CHEAT}
	AddMentalDown( TruePC , 1 );
{$IFDEF PATCH_CHEAT}
	if Check_DrainMaxHP( GB , TruePC , MaxHP_Before , msg ) then begin
		DialogMsg( msg );
	end;
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_GH}
	if (NIL <> GB) then begin
		WaitAMinute( GB , TruePC , ReactionTime( TruePC ) * 5 );
	end;
{$ELSE PATCH_GH}
	WaitAMinute( GB , TruePC , ReactionTime( TruePC ) * 5 );
{$ENDIF PATCH_GH}

	if SkRoll > WreckTarget then begin
		{ First, delink Item from its parent. }
		DelinkGear( Item^.Parent^.SubCom , Item );

		{ Stick the part in the general inventory, if legal. }
		GivePartToPC( GB , Item , PC );

		DoleSkillExperience( TruePC , 31 , 2 );
		DoleExperience( TruePC , 1 );
		it := True;

	end else if SkRoll > SkTarget then begin
		RemoveGear( Item^.Parent^.SubCom , Item );
		Item := Nil;
		it := True;

	end else begin
		it := False;
	end;

	ExtractItem := it;
end;

{$IFDEF PATCH_CHEAT}
Procedure ExtractFrontend( GB: GameBoardPtr; LList, TruePC, PC, Item: GearPtr );
{$ELSE PATCH_CHEAT}
Procedure ExtractFrontend( GB: GameBoardPtr; TruePC , PC , Item: GearPtr );
{$ENDIF PATCH_CHEAT}
	{ Simply remove the provided item. }
	{ PRECOND: PC and ITEM had better be correct, dagnabbit... }
var
	name: String;
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the NIL of GB at here. }
	{ Don't kick out the GG_DisposeGear of LList at here. }
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	name := GearName( Item );
	if GearActive( PC ) then begin
		DialogMsg( MsgString( 'EXTRACT_NOTACTIVE' ) );
{$IFDEF PATCH_CHEAT}
	end else if ExtractItem( GB, LList, TruePC, PC, Item ) then begin
{$ELSE PATCH_CHEAT}
	end else if ExtractItem( GB , TruePC , PC , Item ) then begin
{$ENDIF PATCH_CHEAT}
		if Item = Nil then begin
			DialogMsg( ReplaceHash( MsgString( 'EXTRACT_WRECK' ) , name ) );
		end else begin
			DialogMsg( ReplaceHash( MsgString( 'EXTRACT_OK' ) , name ) );
		end;
	end else begin
		DialogMsg( ReplaceHash( MsgString( 'EXTRACT_FAIL' ) , name ) );
	end;
end;


Procedure EquipItem( GB: GameBoardPtr; PC , Slot , Item: GearPtr );
	{ This is the real equipping procedure. Stuff ITEM into SLOT. }
	{ As noted in TheRules.txt, any nonmaster gear can only have one }
	{ item of any particular "G" type equipped at a time. So, if }
	{ SLOT already has equipment of type ITEM^.G, unequip that and }
	{ stuff it into PC's general inventory. }
var
	I2,I3: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Slot) or (Slot^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ First, check for already equipped items. }
	I2 := Slot^.InvCom;
	while I2 <> Nil do begin
		I3 := I2^.Next;		{ This next step might delink I2, so... }
		if I2^.G = Item^.G then begin
			UnequipItem( GB , PC , I2 );
		end;
		I2 := I3;
	end;

	{ Next, delink Item from PC. }
	DelinkGear( PC^.InvCom , Item );

	{ Next, link ITEM into SLOT. }
	InsertInvCom( Slot , Item );

	{ Equipping an item takes time. }
	if GB <> Nil then WaitAMinute( GB , PC , ReactionTime( PC ) );
end;

Procedure EquipItemFrontend( GB: GameBoardPtr; PC , Item: GearPtr );
	{ Assign ITEM to a legal equipment slot. Move it from the }
	{ general inventory into its new home. }
var
	EI_Menu: RPGMenuPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	ReBrowse: Boolean;
{$ENDIF PATCH_CHEAT}
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Build the slot selection menu. }
	EI_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
{$IFDEF DEBUG}
	MaxNum := BuildSlotMenu( EI_Menu , PC , Item );
{$ELSE DEBUG}
	BuildSlotMenu( EI_Menu , PC , Item );
{$ENDIF DEBUG}
{$IFDEF PATCH_I18N}
	if EI_Menu^.NumItem < 1 then AddRPGMenuItem( EI_Menu , ReplaceHash( I18N_MsgString('EquipItemFrontend','cannot equip'), GearName(Item) ) , -1 );
{$ELSE PATCH_I18N}
	if EI_Menu^.NumItem < 1 then AddRPGMenuItem( EI_Menu , '[cannot equip ' + GearName( Item ) + ']' , -1 );
{$ENDIF PATCH_I18N}

{$IFDEF PATCH_CHEAT}
	repeat
		ReBrowse := False;
  {$IFDEF SDLMODE}
		BP_Source := PC;
		BP_Focus := Item;
		N := SelectMenu( EI_Menu , @FocusOnOneItemRedraw);
  {$ELSE SDLMODE}
		N := SelectMenu( EI_Menu );
  {$ENDIF SDLMODE}

		if N < -1 then begin
  {$IFDEF PATCH_I18N}
			DialogMSG( I18N_MsgString('EquipItemFrontend','You can not install to it') );
  {$ELSE PATCH_I18N}
			DialogMSG( 'You can not install to it.' );
  {$ENDIF PATCH_I18N}
			ReBrowse := True;
		end else if 0 <= N then begin
  {$IFDEF PATCH_I18N}
			DialogMsg( ReplaceHash( I18N_MsgString('EquipItemFrontend','You equip'), GearName(Item) ) );
  {$ELSE PATCH_I18N}
			DialogMsg( 'You equip ' + GearName( Item ) + '.' );
  {$ENDIF PATCH_I18N}
  {$IFDEF DEBUG}
			EquipItem( GB , PC , LocateGearByNumber( PC , N, False, MaxNum, 'EquipItemFrontend' ) , Item );
  {$ELSE DEBUG}
			EquipItem( GB , PC , LocateGearByNumber( PC , N ) , Item );
  {$ENDIF DEBUG}
		end;
	until False = ReBrowse;
	DisposeRPGMenu( EI_Menu );
{$ELSE PATCH_CHEAT}
	{ Select a slot for the item to go into. }
  {$IFDEF SDLMODE}
	BP_Source := PC;
	BP_Focus := Item;
	N := SelectMenu( EI_Menu , @FocusOnOneItemRedraw);
  {$ELSE}
	N := SelectMenu( EI_Menu );
  {$ENDIF}
	DisposeRPGMenu( EI_Menu );

	{ If a slot was selected, pass that info on to the workhorse. }
	if N <> -1 then begin
  {$IFDEF PATCH_I18N}
		DialogMsg( ReplaceHash( I18N_MsgString('EquipItemFrontend','You equip'), GearName(Item) ) );
  {$ELSE PATCH_I18N}
		DialogMsg( 'You equip ' + GearName( Item ) + '.' );
  {$ENDIF PATCH_I18N}
  {$IFDEF DEBUG}
		EquipItem( GB , PC , LocateGearByNumber( PC , N, False, MaxNum, 'EquipItemFrontend' ) , Item );
  {$ELSE DEBUG}
		EquipItem( GB , PC , LocateGearByNumber( PC , N ) , Item );
  {$ENDIF DEBUG}
	end;
{$ENDIF PATCH_CHEAT}
end;

{$IFDEF PATCH_CHEAT}
Function InstallItem( GB: GameBoardPtr; LList, TruePC, Slot: GearPtr; var Item: GearPtr ): Boolean;
{$ELSE PATCH_CHEAT}
Function InstallItem( GB: GameBoardPtr; TruePC , Slot: GearPtr; var Item: GearPtr ): Boolean;
{$ENDIF PATCH_CHEAT}
	{ Attempt the skill rolls needed to install ITEM into the }
	{ requested slot. }
var
	SlotCom,ItemCom,UsedCom: Integer;
	SkTarget,WreckTarget,SkRoll: Integer;
{$IFDEF PATCH_CHEAT}
	MaxHP_Before: LongInt;
	msg: String;
{$ENDIF PATCH_CHEAT}
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the NIL of GB at here. }
	{ Don't kick out the GG_DisposeGear of LList at here. }
	if (NIL = TruePC) or (TruePC^.G <= GG_DisposeGear) then Exit(False);
	if (NIL = Slot) or (Slot^.G <= GG_DisposeGear) then Exit(False);
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}

	{ Error Check - no circular references! }
	if ( FindGearIndex( Item , Slot ) <> -1 ) then Exit( False );

	{ Also, can't engineer things when you're exhausted. }
	if CurrentMental( TruePC ) < 1 then Exit( False );

	{ Can't install into a personal-scale slot. }
{$IFDEF PATCH_CHEAT}
	if (Slot^.Scale = 0) and not(Cheat_MechaCustomize_FreeScale) then Exit( False );
{$ELSE PATCH_CHEAT}
	if Slot^.Scale = 0 then Exit( False );
{$ENDIF PATCH_CHEAT}

	SlotCom := ComponentComplexity( Slot );
	ItemCom := ComponentComplexity( Item );
	UsedCom := SubComComplexity( Slot );

	case Item^.G of
		GG_Weapon,GG_MoveSys: SkTarget := 10;
		GG_Module: SkTarget := 15;
		GG_Sensor: SkTarget := 30;
		GG_Modifier: SkTarget := 25;
	else SkTarget := 20;
	end;
	if Item^.Scale < Slot^.Scale then SkTarget := SkTarget div 2;

	{ The WreckTarget is the target number that must be beat }
	{ in order to avoid accidentally destroying the part... }
	if ( Item^.G = GG_Module ) then begin
		WreckTarget := 8 - Item^.V;
	end else if ( UnscaledMaxDamage( Item ) < 1 ) or ( Item^.Scale < Slot^.Scale ) then begin
		WreckTarget := 7;
	end else begin
		WreckTarget := 10 - UnscaledMaxDamage( Item );
	end;
	if WreckTarget < 3 then WreckTarget := 3;

	{ If the SLOT is going to be overstuffed, better raise the }
	{ number of successes and the target number drastically. }
	if ( ( ItemCom + UsedCom ) > SlotCom ) and ( Not IsMasterGear( Slot ) ) then begin
		SkTarget := SkTarget + ItemCom + UsedCom - SlotCom + 5;
	end;

{$IFDEF PATCH_GH}
	if (NIL <> GB) then begin
		WaitAMinute( GB , TruePC , ReactionTime( TruePC ) * 5 );
	end;
{$ELSE PATCH_GH}
	WaitAMinute( GB , TruePC , ReactionTime( TruePC ) * 5 );
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
	if (NIL <> GB) then begin
		SkRoll := RollStep( TeamSkill( GB , NAV_DefPlayerTeam , 31 ) );
	end else if (NIL <> LList) then begin
		SkRoll := RollStep( TeamSkill( LList, 31 ) );
	end else begin
		SkRoll := RollStep( SkillValue( TruePC, 31 ) );
	end;
{$ELSE PATCH_GH}
	SkRoll := RollStep( TeamSkill( GB , NAV_DefPlayerTeam , 31 ) );
{$ENDIF PATCH_GH}
	if SkRoll > SkTarget then begin
		{ Install the item. }
		DoleSkillExperience( TruePC , 31 , 5 );
		DoleExperience( TruePC , 10 );
		DelinkGear( Item^.Parent^.InvCom , Item );
		InsertSubCom( Slot , Item );
	end else if SkRoll < WreckTarget then begin
		RemoveGear( Item^.Parent^.InvCom , Item );
		Item := Nil;
	end;

{$IFDEF PATCH_CHEAT}
	if GearOperational( TruePC ) then begin
		MaxHP_Before := GearMaxDamage( TruePC );
	end else begin
		MaxHP_Before := 0;
	end;
{$ENDIF PATCH_CHEAT}
	AddMentalDown( TruePC , 1 );
{$IFDEF PATCH_CHEAT}
	if Check_DrainMaxHP( GB , TruePC , MaxHP_Before , msg ) then begin
		DialogMsg( msg );
	end;
{$ENDIF PATCH_CHEAT}
	DoleSkillExperience( TruePC , 31 , 1 );

	InstallItem := SkRoll > SkTarget;
end;

{$IFDEF PATCH_CHEAT}
Procedure InstallFrontend( GB: GameBoardPtr; LList, TruePC, PC, Item: GearPtr );
{$ELSE PATCH_CHEAT}
Procedure InstallFrontend( GB: GameBoardPtr; TruePC , PC , Item: GearPtr );
{$ENDIF PATCH_CHEAT}
	{ Assign ITEM to a legal equipment slot. Move it from the }
	{ general inventory into its new home. }
var
	EI_Menu: RPGMenuPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
	name: String;
{$IFDEF PATCH_CHEAT}
	ReBrowse: Boolean;
{$ENDIF PATCH_CHEAT}
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the NIL of GB at here. }
	{ Don't kick out the GG_DisposeGear of LList at here. }
	if (NIL = TruePC) or (TruePC^.G <= GG_DisposeGear)
		or (NIL = PC) or (PC^.G <= GG_DisposeGear)
		or (NIL = Item) or (Item^.G <= GG_DisposeGear)
	then begin
  {$IFDEF PATCH_I18N}
		DialogMSG( I18N_MsgString('InstallFrontend','You can not install to it') );
  {$ELSE PATCH_I18N}
		DialogMSG( 'You can not install to it.' );
  {$ENDIF PATCH_I18N}
		Exit;
	end;
{$ENDIF PATCH_GH}

	{ Error check- can't install into an active master. }
	if GearActive( PC ) then begin
		DialogMsg( MsgString( 'INSTALL_NOTACTIVE' ) );
		Exit;
	end;

	{ Build the slot selection menu. }
	EI_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
{$IFDEF PATCH_CHEAT}
  {$IFDEF DEBUG}
	MaxNum := BuildSubMenu( EI_Menu , PC , Item , True, Cheat_Install_ShowSubItem );
  {$ELSE DEBUG}
	BuildSubMenu( EI_Menu , PC , Item , True, Cheat_Install_ShowSubItem );
  {$ENDIF DEBUG}
  {$IFDEF PATCH_I18N}
	if EI_Menu^.NumItem < 1 then AddRPGMenuItem( EI_Menu , ReplaceHash( I18N_MsgString('InstallFrontend','cannot install'), GearName(Item) ) , -1 );
  {$ELSE PATCH_I18N}
	if EI_Menu^.NumItem < 1 then AddRPGMenuItem( EI_Menu , '[cannot install ' + GearName( Item ) + ']' , -1 );
  {$ENDIF PATCH_I18N}

	repeat
		ReBrowse := False;
		DialogMsg( GearName( Item ) + ' cmx:' + BStr( ComponentComplexity( Item ) ) + '. ' + MsgSTring( 'BACKPACK_InstallInfo' ) );
  {$IFDEF SDLMODE}
		BP_Source := PC;
		BP_Focus := Item;
		N := SelectMenu( EI_Menu , @FocusOnOneItemRedraw);
  {$ELSE}
		N := SelectMenu( EI_Menu );
  {$ENDIF}

		if N < -1 then begin
  {$IFDEF PATCH_I18N}
			DialogMSG( I18N_MsgString('InstallFrontend','You can not install to it') );
  {$ELSE PATCH_I18N}
			DialogMSG( 'You can not install to it.' );
  {$ENDIF PATCH_I18N}
			ReBrowse := True;
		end else if 0 <= N then begin
			name := GearName( Item );
  {$IFDEF PATCH_CHEAT}
    {$IFDEF DEBUG}
			if InstallItem( GB, LList, TruePC, LocateGearByNumber( PC, N, False, MaxNum, 'InstallFrontend' ), Item ) then begin
    {$ELSE DEBUG}
			if InstallItem( GB, LList, TruePC, LocateGearByNumber( PC, N ), Item ) then begin
    {$ENDIF DEBUG}
  {$ELSE PATCH_CHEAT}
    {$IFDEF DEBUG}
			if InstallItem( GB , TruePC , LocateGearByNumber( PC , N, False, MaxNum, 'InstallFrontend' ) , Item ) then begin
    {$ELSE DEBUG}
			if InstallItem( GB , TruePC , LocateGearByNumber( PC , N ) , Item ) then begin
    {$ENDIF DEBUG}
  {$ENDIF PATCH_CHEAT}
				DialogMsg( ReplaceHash( MsgString( 'INSTALL_OK' ) , name ) );
			end else begin
				if Item = Nil then begin
					DialogMsg( ReplaceHash( MsgString( 'INSTALL_WRECK' ) , name ) );
				end else begin
					DialogMsg( ReplaceHash( MsgString( 'INSTALL_FAIL' ) , name ) );
				end;
			end;
		end;
	until False = ReBrowse;
	DisposeRPGMenu( EI_Menu );
{$ELSE PATCH_CHEAT}
  {$IFDEF DEBUG}
	MaxNum := BuildSubMenu( EI_Menu , PC , Item , True );
  {$ELSE DEBUG}
	BuildSubMenu( EI_Menu , PC , Item , True );
  {$ENDIF DEBUG}
  {$IFDEF PATCH_I18N}
	if EI_Menu^.NumItem < 1 then AddRPGMenuItem( EI_Menu , ReplaceHash( I18N_MsgString('InstallFrontend','cannot install'), GearName(Item) ) , -1 );
  {$ELSE PATCH_I18N}
	if EI_Menu^.NumItem < 1 then AddRPGMenuItem( EI_Menu , '[cannot install ' + GearName( Item ) + ']' , -1 );
  {$ENDIF PATCH_I18N}

	{ Select a slot for the item to go into. }
	DialogMsg( GearName( Item ) + ' cmx:' + BStr( ComponentComplexity( Item ) ) + '. ' + MsgSTring( 'BACKPACK_InstallInfo' ) );
  {$IFDEF SDLMODE}
	BP_Source := PC;
	BP_Focus := Item;
	N := SelectMenu( EI_Menu , @FocusOnOneItemRedraw);
  {$ELSE}
	N := SelectMenu( EI_Menu );
  {$ENDIF}
	DisposeRPGMenu( EI_Menu );

	{ If a slot was selected, pass that info on to the workhorse. }
	if N <> -1 then begin
		{ Store the name here, since the item might get destroyed }
		{ during the installation process. }
		name := GearName( Item );
  {$IFDEF PATCH_CHEAT}
    {$IFDEF DEBUG}
		if InstallItem( GB, LList, TruePC, LocateGearByNumber( PC, N, False, MaxNum, 'InstallFrontend' ), Item ) then begin
    {$ELSE DEBUG}
		if InstallItem( GB, LList, TruePC, LocateGearByNumber( PC, N ), Item ) then begin
    {$ENDIF DEBUG}
  {$ELSE PATCH_CHEAT}
    {$IFDEF DEBUG}
		if InstallItem( GB , TruePC , LocateGearByNumber( PC , N, False, MaxNum, 'InstallFrontend' ) , Item ) then begin
    {$ELSE DEBUG}
		if InstallItem( GB , TruePC , LocateGearByNumber( PC , N ) , Item ) then begin
    {$ENDIF DEBUG}
  {$ENDIF PATCH_CHEAT}
			DialogMsg( ReplaceHash( MsgString( 'INSTALL_OK' ) , name ) );
		end else begin
			if Item = Nil then begin
				DialogMsg( ReplaceHash( MsgString( 'INSTALL_WRECK' ) , name ) );
			end else begin
				DialogMsg( ReplaceHash( MsgString( 'INSTALL_FAIL' ) , name ) );
			end;
		end;
	end;
{$ENDIF PATCH_CHEAT}
end;

{$IFDEF PATCH_GH}
Procedure InstallAmmo( GB: GameBoardPtr; PC , Gun , Ammo: GearPtr );
	{ Place the ammunition gear into the gun. }
var
	A,A2: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Gun) or (Gun^.G <= GG_DisposeGear) then Exit;
	if (NIL = Ammo) or (Ammo^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ To start with, unload any ammo currently in the gun. }
	A := Gun^.SubCom;
	while A <> Nil do begin
		A2 := A^.Next;

{$IFDEF PATCH_GH}
		if (GG_DisposeGear < A^.G) then begin
{$ENDIF PATCH_GH}
		if A^.G = GG_Ammo then begin
			DelinkGear( Gun^.SubCom , A );
			InsertInvCom( PC , A );
		end;
{$IFDEF PATCH_GH}
		end;
{$ENDIF PATCH_GH}

		A := A2;
	end;

	{ Delink the magazine from wherever it currently resides. }
	if IsInvCom( Ammo ) then begin
		DelinkGear( Ammo^.Parent^.InvCom , Ammo );
	end else if IsSubCom( Ammo ) then begin
		DelinkGear( Ammo^.Parent^.SubCom , Ammo );
	end;

	{ Stick the new magazine into the gun. }
	InsertSubCom( Gun , Ammo );

	{ Loading a gun takes time. }
	if GB <> Nil then begin
		WaitAMinute_Part( GB , PC , ReactionTime( PC ) );
	end;
end;

Function NeedAmmo( Weapon: GearPtr ): Boolean;
begin
{$IFDEF PATCH_GH}
	if (NIL = Weapon) or (Weapon^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}
	if ( GG_Weapon <> Weapon^.G ) then exit( False );
	if ( GS_Ballistic <> Weapon^.S ) and ( GS_Missile <> Weapon^.S ) then exit( False );
	NeedAmmo := True;
end;

Function ReloadAmmo( GB: GameBoardPtr; User,Weapon: GearPtr ): Boolean;
var
	Ammo: GearPtr;
	msg: String;
begin
{$IFDEF PATCH_GH}
	if (NIL = User) or (User^.G <= GG_DisposeGear) then Exit(False);
	if (NIL = Weapon) or (Weapon^.G <= GG_DisposeGear) then Exit(False);
{$ENDIF PATCH_GH}
	if ( GG_Weapon <> Weapon^.G ) then exit( False );
	if ( GS_Ballistic <> Weapon^.S ) and ( GS_Missile <> Weapon^.S ) then exit( False );
	if ( NIL <> LocateGoodAmmo( Weapon ) ) then exit( True );

	Ammo := SearchGoodAmmo( User , Weapon );
	if ( NIL <> Ammo ) then begin
		DialogMsg( ReplaceHash( I18N_MsgString('ReloadAmmo','Ammo loaded'), GearName( LocatePilot( User ) ) , GearName(Weapon) , GearName(Ammo) ) );

		{ Loading a gun takes time. }
		InstallAmmo( GB , User , Weapon , Ammo );
	end;
	ReloadAmmo := ( NIL <> LocateGoodAmmo( Weapon ) );
end;
{$ELSE PATCH_GH}
Procedure InstallAmmo( GB: GameBoardPtr; PC , Gun , Ammo: GearPtr );
	{ Place the ammunition gear into the gun. }
var
	A,A2: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Gun) or (Gun^.G <= GG_DisposeGear) then Exit;
	if (NIL = Ammo) or (Ammo^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ To start with, unload any ammo currently in the gun. }
	A := Gun^.SubCom;
	while A <> Nil do begin
		A2 := A^.Next;

{$IFDEF PATCH_GH}
		if (GG_DisposeGear < A^.G) then begin
{$ENDIF PATCH_GH}
		if A^.G = GG_Ammo then begin
			DelinkGear( Gun^.SubCom , A );
			InsertInvCom( PC , A );
		end;
{$IFDEF PATCH_GH}
		end;
{$ENDIF PATCH_GH}

		A := A2;
	end;

	{ Delink the magazine from wherever it currently resides. }
	if IsInvCom( Ammo ) then begin
		DelinkGear( Ammo^.Parent^.InvCom , Ammo );
	end else if IsSubCom( Ammo ) then begin
		DelinkGear( Ammo^.Parent^.SubCom , Ammo );
	end;

	{ Stick the new magazine into the gun. }
	InsertSubCom( Gun , Ammo );

	{ Loading a gun takes time. }
	if GB <> Nil then WaitAMinute( GB , PC , ReactionTime( PC ) );
end;
{$ENDIF PATCH_GH}

Procedure InstallAmmoFrontend( GB: GameBoardPtr; PC , Item: GearPtr );
	{ Assign ITEM to a legal projectile weapon. Move it from the }
	{ general inventory into its new home. }
var
	IA_Menu: RPGMenuPtr;
	Gun: GearPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	ReBrowse: Boolean;
{$ENDIF PATCH_CHEAT}
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Build the slot selection menu. }
	IA_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
{$IFDEF PATCH_CHEAT}
  {$IFDEF DEBUG}
	MaxNum := BuildSubMenu( IA_Menu , PC , Item , False, Cheat_InstallAmmo_ShowSubItem );
  {$ELSE DEBUG}
	BuildSubMenu( IA_Menu , PC , Item , False, Cheat_InstallAmmo_ShowSubItem );
  {$ENDIF DEBUG}
  {$IFDEF PATCH_I18N}
	if IA_Menu^.NumItem < 1 then AddRPGMenuItem( IA_Menu , ReplaceHash( I18N_MsgString('InstallAmmoFrontend','no weapon'), GearName(Item) ) , -1 );
  {$ELSE PATCH_I18N}
	if IA_Menu^.NumItem < 1 then AddRPGMenuItem( IA_Menu , '[no weapon for ' + GearName( Item ) + ']' , -1 );
  {$ENDIF PATCH_I18N}

	repeat
		ReBrowse := False;
  {$IFDEF SDLMODE}
		BP_Source := PC;
		BP_Focus := Item;
		N := SelectMenu( IA_Menu , @FocusOnOneItemRedraw);
  {$ELSE}
		N := SelectMenu( IA_Menu );
  {$ENDIF}

		if N < -1 then begin
  {$IFDEF PATCH_I18N}
			DialogMSG( I18N_MsgString('InstallAmmoFrontend','You can not install to it') );
  {$ELSE PATCH_I18N}
			DialogMSG( 'You can not install to it.' );
  {$ENDIF PATCH_I18N}
			ReBrowse := True;
		end else if 0 <= N then begin
  {$IFDEF DEBUG}
			Gun := LocateGearByNumber( PC , N, False, MaxNum, 'InstallAmmoFrontend' );
  {$ELSE DEBUG}
			Gun := LocateGearByNumber( PC , N );
  {$ENDIF DEBUG}
  {$IFDEF PATCH_I18N}
			DialogMsg( ReplaceHash( I18N_MsgString('InstallAmmoFrontend','You load'), GearName(Gun), GearName(Item) ) );
  {$ELSE PATCH_I18N}
			DialogMsg( 'You load ' + GearName( Item ) + ' into ' + GearName( Gun ) + '.' );
  {$ENDIF PATCH_I18N}
			InstallAmmo( GB , PC , Gun , Item );
		end;
	until False = ReBrowse;
	DisposeRPGMenu( IA_Menu );
{$ELSE PATCH_CHEAT}
  {$IFDEF DEBUG}
	MaxNum := BuildSubMenu( IA_Menu , PC , Item , False );
  {$ELSE DEBUG}
	BuildSubMenu( IA_Menu , PC , Item , False );
  {$ENDIF DEBUG}
  {$IFDEF PATCH_I18N}
	if IA_Menu^.NumItem < 1 then AddRPGMenuItem( IA_Menu , ReplaceHash( I18N_MsgString('InstallAmmoFrontend','no weapon'), GearName(Item) ) , -1 );
  {$ELSE PATCH_I18N}
	if IA_Menu^.NumItem < 1 then AddRPGMenuItem( IA_Menu , '[no weapon for ' + GearName( Item ) + ']' , -1 );
  {$ENDIF PATCH_I18N}

	{ Select a slot for the item to go into. }
  {$IFDEF SDLMODE}
	BP_Source := PC;
	BP_Focus := Item;
	N := SelectMenu( IA_Menu , @FocusOnOneItemRedraw);
  {$ELSE}
	N := SelectMenu( IA_Menu );
  {$ENDIF}
	DisposeRPGMenu( IA_Menu );

	{ If a slot was selected, pass that info on to the workhorse. }
	if N <> -1 then begin
  {$IFDEF DEBUG}
		Gun := LocateGearByNumber( PC , N, False, MaxNum, 'InstallAmmoFrontend' );
  {$ELSE DEBUG}
		Gun := LocateGearByNumber( PC , N );
  {$ENDIF DEBUG}
  {$IFDEF PATCH_I18N}
		DialogMsg( ReplaceHash( I18N_MsgString('InstallAmmoFrontend','You load'), GearName(Gun), GearName(Item) ) );
  {$ELSE PATCH_I18N}
		DialogMsg( 'You load ' + GearName( Item ) + ' into ' + GearName( Gun ) + '.' );
  {$ENDIF PATCH_I18N}
		InstallAmmo( GB , PC , Gun , Item );
	end;
{$ENDIF PATCH_CHEAT}
end;


Procedure DropFrontEnd( PC , Item: GearPtr );
	{ How to drop an item: Make sure PC is a root-level gear. }
	{ Delink ITEM from its current location. }
	{ Copy PC's location variables to ITEM. }
	{ Install ITEM as the next sibling of PC. }
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Make sure PC is at root level... }
	PC := FindRoot( PC );
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Delink ITEM from its parent... }
	DelinkGear( Item^.Parent^.InvCom , Item );

	{ Copy the location variables to ITEM... }
	SetNAtt( Item^.NA , NAG_Location , NAS_X , NAttValue( PC^.NA , NAG_Location , NAS_X ) );
	SetNAtt( Item^.NA , NAG_Location , NAS_Y , NAttValue( PC^.NA , NAG_Location , NAS_Y ) );
	if not OnTheMap( PC ) then SetNAtt( Item^.NA , NAG_Location , NAS_Team , NAV_DefPlayerTeam );

	{ Install ITEM as PC's sibling... }
	Item^.Next := PC^.Next;
	PC^.Next := Item;

	{ Do display stuff. }
{$IFDEF PATCH_I18N}
	DialogMsg( ReplaceHash( I18N_MsgString('DropFrontEnd','You drop'), GearName(Item) ) );
{$ELSE PATCH_I18N}
	DialogMsg( 'You drop ' + GearName( Item ) + '.' );
{$ENDIF PATCH_I18N}
end;

Procedure DropAllFrontEnd( GB: GameBoardPtr; var LList: GearPtr; PC , Part: GearPtr );
var
	RPM: RPGMenuPtr;
	N: Integer;
	Part_Next: GearPtr;
begin
{$IFDEF SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_YesNoMenu );
{$ELSE SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_Menu );
{$ENDIF SDLMODE}
{$IFDEF PATCH_I18N}
	AddRPGMenuItem( RPM , I18N_MsgString('DropAllFrontEnd','Cancel') , -1 );
	AddRPGMenuItem( RPM , I18N_MsgString('DropAllFrontEnd','OK') , 1 );
{$ELSE PATCH_I18N}
	AddRPGMenuItem( RPM , '[Cancel]' , -1 );
	AddRPGMenuItem( RPM , '[OK]' , 1 );
{$ENDIF PATCH_I18N}
	RPM^.Mode := RPMNormal;
{$IFDEF SDLMODE}
	InfoBox( ZONE_YesNoMenu.GetRect() );
	N := SelectMenu( RPM , NIL );
{$ELSE SDLMODE}
	N := SelectMenu( RPM );
{$ENDIF SDLMODE}
	DisposeRPGMenu( RPM );
	if ( -1 = N ) then exit;

	while ( NIL <> Part ) do begin
		Part_Next := Part^.Next;
		if CheckAlongPath_DisallowDropping( Part ) then begin
		end else begin
			DropFrontEnd( PC , Part );
		end;
		Part := Part_Next;
	end;
end;

Procedure DropEqpAllFrontEnd( GB: GameBoardPtr; var LList: GearPtr; PC , Master: GearPtr );
	Procedure CheckAlongPath( Part: GearPtr; IsInv: Boolean );
	var
		Part_Next: GearPtr;
	begin
		while ( NIL <> Part ) do begin
			Part_Next := Part^.Next;
			if IsInv and ( not CheckAlongPath_DisallowDropping( Part ) ) then begin
				DropFrontEnd( PC , Part );
			end;
			CheckAlongPath( Part^.InvCom , True );
			CheckAlongPath( Part^.SubCom , False );
			Part := Part_Next;
		end;
	end;
var
	RPM: RPGMenuPtr;
	N: Integer;
begin
{$IFDEF SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_YesNoMenu );
{$ELSE SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_Menu );
{$ENDIF SDLMODE}
{$IFDEF PATCH_I18N}
	AddRPGMenuItem( RPM , I18N_MsgString('DropAllFrontEnd','Cancel') , -1 );
	AddRPGMenuItem( RPM , I18N_MsgString('DropAllFrontEnd','OK') , 1 );
{$ELSE PATCH_I18N}
	AddRPGMenuItem( RPM , '[Cancel]' , -1 );
	AddRPGMenuItem( RPM , '[OK]' , 1 );
{$ENDIF PATCH_I18N}
	RPM^.Mode := RPMNormal;
{$IFDEF SDLMODE}
	InfoBox( ZONE_YesNoMenu.GetRect() );
	N := SelectMenu( RPM , NIL );
{$ELSE SDLMODE}
	N := SelectMenu( RPM );
{$ENDIF SDLMODE}
	DisposeRPGMenu( RPM );
	if ( -1 = N ) then exit;

	CheckAlongPath( Master^.InvCom , False );
	CheckAlongPath( Master^.SubCom , False );
end;

{$IFDEF PATCH_GH}
Function TradeFrontEnd_GetDest( GB: GameBoardPtr; PC , Item, LList: GearPtr ): GearPtr;
const
	Unsafe_Transfer_Range = 10;
var
	TI_Menu: RPGMenuPtr;
	M: GearPtr;
	Team: Integer;
	N: LongInt;
	X,Y: Integer;

	Function Transferable_To ( Dest: GearPtr ) : Boolean;
	var
		DTeam, DX, DY : Integer;
	begin
		If Dest = PC then Exit(False);

		{ Team check.  This could probably be simplified --
		  How could the source Master's team be other than
		  DefPlayer or Lancemate anyway? }
		DTeam := NAttValue( Dest^.NA , NAG_Location , NAS_Team );
		if DTeam = NAV_LancemateTeam then DTeam := NAV_DefPlayerTeam;
		if DTeam <> Team then Exit(False);

		if X = 0 then Exit(True); {safe area case}

		{we're now in the unsafe area case, check for adjacency}
		DX := NAttValue( Dest^.NA, NAG_Location, NAS_X);
		DY := NAttValue( Dest^.NA, NAG_Location, NAS_Y);
		if not OnTheMap(DX,DY) then Exit(False); {cannot transfer to offmap stuff}

		Transferable_To := ( Range(X,Y,DX,DY) <= Unsafe_Transfer_Range );
	end;
begin
	if ( IsSafeArea( GB ) or ( NIL = GB ) ) then begin
		X := 0;
		Y := 0;
	end else begin
		DialogMsg( MsgString( 'TRANSFER_UNSAFE' ) );
		X := NAttValue(PC^.NA, NAG_Location, NAS_X);
		Y := NAttValue(PC^.NA, NAG_Location, NAS_Y);
	end;

	{ Build the slot selection menu. }
  {$IFDEF SDLMODE}
	TI_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_FHQMenu );
  {$ELSE}
	TI_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
  {$ENDIF}
	N := 1;
	M := LList;
	Team := NAttValue( PC^.NA , NAG_Location , NAS_Team );
	if Team = NAV_LancemateTeam then Team := NAV_DefPlayerTeam;

	{ This menu should contain all the masters from LList which }
	{ belong to Team 1. }
	while ( NIL <> M ) do begin
		if (GG_DisposeGear < M^.G) then begin
  {$IFDEF PATCH_CHEAT}
			if IsMasterGear( M ) and (Cheat_Trade_NotSafeArea or Transferable_To(M)) then begin
				if Cheat_Trade_LanceMateMenuName then begin
					AddRPGMenuItem( TI_Menu , LanceMateMenuName( GB , M ) , N );
				end else begin
					AddRPGMenuItem( TI_Menu , GearName( M ) , N );
				end;
			end;
  {$ELSE PATCH_CHEAT}
			if IsMasterGear( M ) and Transferable_To(M) then begin
				AddRPGMenuItem( TI_Menu , GearName( M ) , N );
			end;
  {$ENDIF PATCH_CHEAT}
			Inc( N );
		end;
		M := M^.Next;
	end;
	AlphaKeyMenu( TI_Menu );

  {$IFDEF PATCH_I18N}
	if TI_Menu^.NumItem < 1 then AddRPGMenuItem( TI_Menu , I18N_MsgString( 'TradeFrontEnd' , 'CannotTrade' ) , -1 );
  {$ELSE PATCH_I18N}
	if TI_Menu^.NumItem < 1 then AddRPGMenuItem( TI_Menu , '[cannot trade ' + GearName( Item ) + ']' , -1 );
  {$ENDIF PATCH_I18N}

	{ Select a slot for the item to go into. }
  {$IFDEF SDLMODE}
	InfoGear := Item;
	BP_Source := LList;
	BP_ActiveMenu := TI_Menu;
	if ( NIL = Item ) then begin
		BPRD_Caption := I18N_MsgString( 'TradeFrontEnd' , 'AllGiveTo' );
	end else begin
		BPRD_Caption := ReplaceHash( I18N_MsgString( 'TradeFrontEnd' , 'GiveTo' ), GearName( Item ) );
	end;

	N := SelectMenu( TI_Menu , @TransferRedraw);
  {$ELSE}
	N := SelectMenu( TI_Menu );
  {$ENDIF}
	DisposeRPGMenu( TI_Menu );

	TradeFrontEnd_GetDest := RetrieveGearSib( LList , N );
end;
{$ENDIF PATCH_GH}

Procedure TradeFrontend( GB: GameBoardPtr; PC , Item, LList: GearPtr );
	{ Assign ITEM to a different master. Move it from the }
	{ general inventory of PC into its new home. }
const
	Unsafe_Transfer_Range = 10;
var
{$IFDEF PATCH_GH}
	M: GearPtr;
{$ELSE PATCH_GH}
	TI_Menu: RPGMenuPtr;
	M: GearPtr;
	Team,N: Integer;
	X,Y: Integer;

	Function Transferable_To ( Dest: GearPtr ) : Boolean;
	var
		DTeam, DX, DY : Integer;
	begin
		If Dest = PC then Exit(False);

		{ Team check.  This could probably be simplified --
		  How could the source Master's team be TradeFrontendother than
		  DefPlayer or Lancemate anyway? }
		DTeam := NAttValue( Dest^.NA , NAG_Location , NAS_Team );
		if DTeam = NAV_LancemateTeam then DTeam := NAV_DefPlayerTeam;
		if DTeam <> Team then Exit(False);

		if X = 0 then Exit(True); {safe area case}

		{we're now in the unsafe area case, check for adjacency}
		DX := NAttValue( Dest^.NA, NAG_Location, NAS_X);
		DY := NAttValue( Dest^.NA, NAG_Location, NAS_Y);
		if not OnTheMap(DX,DY) then Exit(False); {cannot transfer to offmap stuff}

		Transferable_To := ( Range(X,Y,DX,DY) <= Unsafe_Transfer_Range );
	end;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
	{ Don't kick out the GG_DisposeGear of LList at here. }
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
	M := TradeFrontEnd_GetDest( GB , PC , Item, LList );

	{ If a slot was selected, pass that info on to the workhorse. }
	if ( NIL <> M ) then begin
		if IsLegalSlot( M , Item ) then begin
			DelinkGear( Item^.Parent^.InvCom , Item );
			InsertInvCom( M , Item );
			DialogMsg( MsgString( 'BACKPACK_ItemTraded' ) );
		end else begin
			DialogMsg( MsgString( 'BACKPACK_NotTraded' ) );
		end;
	end;
{$ELSE PATCH_GH}
	if IsSafeArea( GB ) then begin
		X := 0;
		Y := 0;
	end else begin
		DialogMsg( MsgString( 'TRANSFER_UNSAFE' ) );
		X := NAttValue(PC^.NA, NAG_Location, NAS_X);
		Y := NAttValue(PC^.NA, NAG_Location, NAS_Y);
	end;

	{ Build the slot selection menu. }
{$IFDEF SDLMODE}
	TI_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_FHQMenu );
{$ELSE}
	TI_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
{$ENDIF}
	N := 1;
	M := LList;
	Team := NAttValue( PC^.NA , NAG_Location , NAS_Team );
	if Team = NAV_LancemateTeam then Team := NAV_DefPlayerTeam;

	{ This menu should contain all the masters from LList which }
	{ belong to Team 1. }
	while M <> Nil do begin
{$IFDEF PATCH_CHEAT}
		if IsMasterGear( M ) and (Cheat_Trade_NotSafeArea or Transferable_To(M)) then begin
			AddRPGMenuItem( TI_Menu , LanceMateMenuName( M ) , N );
		end;
{$ELSE PATCH_CHEAT}
		if IsMasterGear( M ) and Transferable_To(M) then
			AddRPGMenuItem( TI_Menu , LanceMateMenuName( M ) , N );
{$ENDIF PATCH_CHEAT}
		M := M^.Next;
		Inc( N );
	end;
	AlphaKeyMenu( TI_Menu );

{$IFDEF PATCH_I18N}
	if TI_Menu^.NumItem < 1 then AddRPGMenuItem( TI_Menu , ReplaceHash( I18N_MsgString('TradeFrontend','cannot trade'), GearName(Item) ) , -1 );
{$ELSE PATCH_I18N}
	if TI_Menu^.NumItem < 1 then AddRPGMenuItem( TI_Menu , '[cannot trade ' + GearName( Item ) + ']' , -1 );
{$ENDIF PATCH_I18N}

	{ Select a slot for the item to go into. }
{$IFDEF SDLMODE}
	InfoGear := Item;
	BP_Source := LList;
	BP_ActiveMenu := TI_Menu;
	BPRD_Caption := ReplaceHash( MsgString('TRANSFER_GiveTo'), GearName( Item ) );

	N := SelectMenu( TI_Menu , @TransferRedraw);
{$ELSE}
	N := SelectMenu( TI_Menu );
{$ENDIF}
	DisposeRPGMenu( TI_Menu );

	{ If a slot was selected, pass that info on to the workhorse. }
	if N <> -1 then begin
		M := RetrieveGearSib( LList , N );
		if IsLegalSlot( M , Item ) then begin
			DelinkGear( Item^.Parent^.InvCom , Item );
			InsertInvCom( M , Item );
			DialogMsg( MsgString( 'BACKPACK_ItemTraded' ) );
		end else begin
			DialogMsg( MsgString( 'BACKPACK_NotTraded' ) );
		end;
	end;
{$ENDIF PATCH_GH}
end;

{$IFDEF PATCH_GH}
Procedure TradeAllFrontend( GB: GameBoardPtr; PC , Item, LList: GearPtr );
	{ Assign ITEM to a different master. Move it from the }
	{ general inventory of PC into its new home. }
var
	M: GearPtr;
	Item_Next: GearPtr;
begin
	M := TradeFrontEnd_GetDest( GB , PC , NIL, LList );

	if ( NIL <> M ) then begin
		while ( NIL <> Item ) do begin
			Item_Next := Item^.Next;
			if CheckAlongPath_DisallowTransfering( Item ) then begin
			end else begin
				if IsLegalSlot( M , Item ) then begin
					DelinkGear( Item^.Parent^.InvCom , Item );
					InsertInvCom( M , Item );
  {$IFDEF PATCH_I18N}
					DialogMsg( ReplaceHash( I18N_MsgString( 'TradeAllFrontend' , 'ItemTraded' ) , GearName( Item ) ) );
  {$ELSE PATCH_I18N}
					DialogMsg( 'Item traded.' );
  {$ENDIF PATCH_I18N}
				end else begin
  {$IFDEF PATCH_I18N}
					DialogMsg( ReplaceHash( I18N_MsgString( 'TradeAllFrontend' , 'NotTraded' ) , GearName( Item ) ) );
  {$ELSE PATCH_I18N}
					DialogMsg( 'Not traded.' );
  {$ENDIF PATCH_I18N}
				end;
			end;
			Item := Item_Next;
		end;
	end;
end;

Procedure TradeEqpAllFrontEnd( GB: GameBoardPtr; PC , Item, LList: GearPtr );
	{ Assign ITEM to a different master. Move it from the }
	{ general inventory of PC into its new home. }
var
	M: GearPtr;

	Procedure CheckAlongPath( Part: GearPtr; IsInv: Boolean );
	var
		Part_Next: GearPtr;
	begin
		while ( NIL <> Part ) do begin
			Part_Next := Part^.Next;
			if IsInv and ( not CheckAlongPath_DisallowTransfering( Part ) ) then begin
				if IsLegalSlot( M , Part ) then begin
					DelinkGear( Part^.Parent^.InvCom , Part );
					InsertInvCom( M , Part );
  {$IFDEF PATCH_I18N}
					DialogMsg( ReplaceHash( I18N_MsgString( 'TradeAllFrontend' , 'ItemTraded' ) , GearName( Part ) ) );
  {$ELSE PATCH_I18N}
					DialogMsg( 'Item traded.' );
  {$ENDIF PATCH_I18N}
				end else begin
  {$IFDEF PATCH_I18N}
					DialogMsg( ReplaceHash( I18N_MsgString( 'TradeAllFrontend' , 'NotTraded' ) , GearName( Part ) ) );
  {$ELSE PATCH_I18N}
					DialogMsg( 'Not traded.' );
  {$ENDIF PATCH_I18N}
				end;
			end;
			CheckAlongPath( Part^.InvCom , True );
			CheckAlongPath( Part^.SubCom , False );
			Part := Part_Next;
		end;
	end;
begin
	M := TradeFrontEnd_GetDest( GB , PC , NIL, LList );

	if ( NIL <> M ) then begin
		CheckAlongPath( Item^.InvCom , False );
		CheckAlongPath( Item^.SubCom , False );
	end;
end;
{$ENDIF PATCH_GH}

Procedure FHQ_AssociatePilotMek( PC , M , LList: GearPtr );
	{ Associate the mecha with the pilot. }
begin
	AssociatePilotMek( LList , PC , M );
	DialogMsg( ReplaceHash( MsgString( 'FHQ_AssociatePM' ) , GearName( PC ) ) );
end;

{$IFDEF SDLMODE}
Procedure SelectPilotForMechaRedraw;
	{ Do a redraw for the Field HQ. }
var
	Part: GearPtr;
	N: Integer;
begin
	if InfoGB <> Nil then SDLCombatDisplay( InfoGB );
	if BPRD_CAPTION <> '' then begin
		InfoBox( ZONE_FHQTitle.GetRect() );
		CMessage( BPRD_CAPTION , ZONE_FHQTitle.GetRect() , InfoHilight );
	end;
	InfoBox( ZONE_FHQMenu1.GetRect() );
	InfoBox( ZONE_FHQMenu2.GetRect() );
	InfoBox( ZONE_FHQInfo.GetRect() );
	MechaInfoForSelectingAPilot( BP_Focus, InfoGB, ZONE_FHQMenu1 );
	if ( BP_ActiveMenu <> Nil ) and ( BP_Source <> Nil ) then begin
		N := CurrentMenuItemValue( BP_ActiveMenu );
		if N > 0 then begin
			Part := RetrieveGearSib( BP_Source , N );
			if Part <> Nil then begin
				LongformGearInfo( Part , InfoGB, ZONE_FHQInfo );
			end;
		end;
	end;
end;
{$ENDIF}

Procedure FHQ_SelectPilotForMecha( GB: GameBoardPtr; Mek: GearPtr );
	{ Select a pilot for the mecha in question. }
	{ Pilots must be characters- they must either belong to the default }
	{ player team or, if they're lancemates, they must have a CID. }
	{ This is to prevent the PC from dominating some sewer rats and }
	{ training them to be pilots. }
var
	RPM: RPGMenuPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
	M: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Create the menu. }
{$IFDEF SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_FHQMenu2 );
{$ELSE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_Menu );
{$ENDIF}
	M := GB^.Meks;
	N := 1;
	while M <> Nil do begin
{$IFDEF PATCH_GH}
		if (GG_DisposeGear < M^.G) then begin
			if M^.G = GG_Character then begin
				if ( NAttValue( M^.NA , NAG_Location , NAS_Team ) = NAV_LancemateTeam ) and ( NAttValue( M^.NA , NAG_Personal , NAS_CID ) <> 0 ) then begin
					AddRPGMenuItem( RPM , GearName( M ) , N );
				end else if NAttValue( M^.NA , NAG_Location , NAS_Team ) = NAV_DefPlayerTeam then begin
					AddRPGMenuItem( RPM , GearName( M ) , N );
				end;
			end;
			Inc( N );
		end;
		M := M^.Next;
{$ELSE PATCH_GH}
		if M^.G = GG_Character then begin
			if ( NAttValue( M^.NA , NAG_Location , NAS_Team ) = NAV_LancemateTeam ) and ( NAttValue( M^.NA , NAG_Personal , NAS_CID ) <> 0 ) then begin
				AddRPGMenuItem( RPM , GearName( M ) , N );
			end else if NAttValue( M^.NA , NAG_Location , NAS_Team ) = NAV_DefPlayerTeam then begin
				AddRPGMenuItem( RPM , GearName( M ) , N );
			end;
		end;
		M := M^.Next;
		Inc( N );
{$ENDIF PATCH_GH}
	end;
	RPMSortAlpha( RPM );
	AddRPGMenuItem( RPM , MSgString( 'EXIT' ) , -1 );

	{ Get a selection from the menu. }
{$IFDEF SDLMODE}
	BP_Focus := Mek;
	BP_Source := GB^.Meks;
	INFOGB := GB;
	BPRD_CAPTION := ReplaceHash( MsgString('FIELDHQ_SELECTPILOTFORMECHA'), GearName(Mek) );
	BP_ActiveMenu := RPM;

	n := SelectMenu( RPM , @SelectPilotForMechaRedraw );
{$ELSE}
  {$IFDEF PATCH_CHEAT}
	InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
	n := SelectMenu( RPM );
{$ENDIF}
	DisposeRPGMenu( RPM );

	if N > 0 then begin
		M := RetrieveGearSib( GB^.Meks , N );
		FHQ_AssociatePilotMek( M , Mek , GB^.Meks );
	end;
end;

{$IFDEF SDLMODE}
Procedure SelectMechaForPilotRedraw;
	{ Do a redraw for the Field HQ. }
var
	Part: GearPtr;
	N: Integer;
begin
	if InfoGB <> Nil then SDLCombatDisplay( InfoGB );
	if BPRD_CAPTION <> '' then begin
		InfoBox( ZONE_FHQTitle.GetRect() );
		CMessage( BPRD_CAPTION , ZONE_FHQTitle.GetRect() , InfoHilight );
	end;
	InfoBox( ZONE_FHQMenu1.GetRect() );
	InfoBox( ZONE_FHQMenu2.GetRect() );
	InfoBox( ZONE_FHQInfo.GetRect() );
	PilotInfoForSelectingAMecha( BP_Focus, InfoGB, ZONE_FHQMenu1 );
	if ( BP_ActiveMenu <> Nil ) and ( BP_Source <> Nil ) then begin
		N := CurrentMenuItemValue( BP_ActiveMenu );
		if N > 0 then begin
			Part := RetrieveGearSib( BP_Source , N );
			if Part <> Nil then begin
				LongformGearInfo( Part , InfoGB, ZONE_FHQInfo );
			end;
		end;
	end;
end;
{$ENDIF}

Procedure FHQ_SelectMechaForPilot( GB: GameBoardPtr; NPC: GearPtr );
	{ Select a pilot for the mecha in question. }
	{ Pilots must be characters- they must either belong to the default }
	{ player team or, if they're lancemates, they must have a CID. }
	{ This is to prevent the PC from dominating some sewer rats and }
	{ training them to be pilots. }
var
	RPM: RPGMenuPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
	M: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = NPC) or (NPC^.G <= GG_DisposeGear) then begin
  {$IFDEF SDLMODE}
		INFOGear := NIL;
  {$ENDIF}
		Exit;
	end;
{$ENDIF PATCH_GH}
{$IFDEF SDLMODE}
	BP_Focus := NPC;
	BP_Source := GB^.Meks;
	INFOGB := GB;
	BPRD_CAPTION := ReplaceHash( MsgString('FIELDHQ_SELECTMECHAFORPILOT'), GearName(NPC) );
{$ELSE}
  {$IFDEF PATCH_CHEAT}
	InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
{$ENDIF}

	{ Error check- only characters can pilot mecha! Pets can't. }
	if ( NAttValue( NPC^.NA , NAG_Personal , NAS_CID ) = 0 ) and ( NAttValue( NPC^.NA , NAG_Location , NAS_Team ) <> NAV_DefPlayerTeam ) then begin
		DialogMsg( ReplaceHash( MsgString( 'FHQ_SMFP_NoPets' ) , GearName( NPC ) ) );
		Exit;
	end;

	{ Create the menu. }
{$IFDEF SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_FHQMenu2 );
{$ELSE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_Menu );
{$ENDIF}
	M := GB^.Meks;
	N := 1;
	while M <> Nil do begin
{$IFDEF PATCH_GH}
		if (GG_DisposeGear < M^.G) then begin
			if ( M^.G = GG_Mecha ) and ( NAttValue( M^.NA , NAG_Location , NAS_Team ) = NAV_DefPlayerTeam ) then begin
				AddRPGMenuItem( RPM , LanceMateMenuName( GB , M ) , N );
			end;
			Inc( N );
		end;
		M := M^.Next;
{$ELSE PATCH_GH}
		if ( M^.G = GG_Mecha ) and ( NAttValue( M^.NA , NAG_Location , NAS_Team ) = NAV_DefPlayerTeam ) then begin
			AddRPGMenuItem( RPM , LanceMateMenuName( M ) , N );
		end;
		M := M^.Next;
		Inc( N );
{$ENDIF PATCH_GH}
	end;
	RPMSortAlpha( RPM );
	AddRPGMenuItem( RPM , MSgString( 'EXIT' ) , -1 );

	{ Get a selection from the menu. }
{$IFDEF SDLMODE}
	BP_ActiveMenu := RPM;
	n := SelectMenu( RPM , @SelectMechaForPilotRedraw );
{$ELSE}
	n := SelectMenu( RPM );
{$ENDIF}
	DisposeRPGMenu( RPM );

	if N > 0 then begin
		M := RetrieveGearSib( GB^.Meks , N );
		FHQ_AssociatePilotMek( NPC , M , GB^.Meks );
	end;
end;

Procedure StartContinuousUseItem( GB: GameBoardPtr; TruePC , Item: GearPtr );
	{ The PC wants to use this item. Give it a try. }
var
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if (NIL = TruePC) or (TruePC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Find the item's index number. If the item cannot be found }
	{ on the TRUEPC, then this item cannot be used. }
	N := FindGearIndex( TruePC , Item );
	if N > 0 then begin
		WaitAMinute( GB , TruePC , 1 );
		SetNAtt( TruePC^.NA , NAG_Location , NAS_SmartAction , NAV_UseItem );
		SetNAtt( TruePC^.NA , NAG_Location , NAS_SmartWeapon , N );
		SetNAtt( TruePC^.NA , NAG_Location , NAS_SmartCount , 3 );

		{ When an item is used in this way, exit the menu. }
		ForceQuit := True;
	end else begin
		DialogMsg( MsgString( 'BACKPACK_CantUse' ) );
	end;
end;

Procedure UseScriptItem( GB: GameBoardPtr; TruePC, Item: GearPtr; T: String );
	{ This item has a script effect. Exit the backpack and use it. }
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the NIL of GB at here. }
	{ Don't kick out the GG_DisposeGear of TruePC at here. }
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	if SAttValue( Item^.SA , T ) <> '' then begin
		{ Announce the intention. }
		DialogMsg( ReplaceHash( MsgString( 'BACKPACK_Script_' + T ) , GearName( Item ) ) );

		{ Using items takes time... }
{$IFDEF PATCH_GH}
		if (NIL <> TruePC) and (GG_DisposeGear < TruePC^.G) then begin
			WaitAMinute( GB , TruePC , ReactionTime( TruePC ) );
		end;
{$ELSE PATCH_GH}
		WaitAMinute( GB , TruePC , ReactionTime( TruePC ) );
{$ENDIF PATCH_GH}

		{ ...and also exits the backpack. }
		ForceQuit := True;
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_GH}
		if (NIL <> GB) then begin
			GFCombatDisplay( GB );
		end;
  {$ELSE PATCH_GH}
		GFCombatDisplay( GB );
  {$ENDIF PATCH_GH}
{$ENDIF}

		{ Finally, trigger the script. }
		TriggerGearScript( GB , Item , T );
	end else begin
		{ Announce the lack of a valid script. }
		DialogMsg( ReplaceHash( MsgString( 'BACKPACK_CannotUseScript' ) , GearName( Item ) ) );
	end;
end;

{$IFDEF PATCH_CHEAT}
Procedure UseSkillOnItem( GB: GameBoardPtr; LList, TruePC, Item: GearPtr );
{$ELSE PATCH_CHEAT}
Procedure UseSkillOnItem( GB: GameBoardPtr; TruePC, Item: GearPtr );
{$ENDIF PATCH_CHEAT}
	{ The PC will have the option to use a CLUE-type skill on this }
	{ item, maybe to gain some new information, activate an effect, }
	{ or whatever else. }
var
	SkMenu: RPGMenuPtr;
	T: Integer;
	msg: String;
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the NIL of GB at here. }
	{ Don't kick out the GG_DisposeGear of LList at here. }
	{ Don't kick out the GG_DisposeGear of TruePC at here. }
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	SkMenu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );

	{ Add the usable skills. }
	for t := 1 to NumSkill do begin
		{ In order to be usable, it must be a CLUE type skill, }
		{ and the PC must have ranks in it. }
{$IFDEF PATCH_CHEAT}
		if ((USAGE_Clue = SkillMan[T].Usage)
			and (
				((NIL <> GB) and TeamHasSkill( GB, NAV_DefPlayerTeam, T ))
				or ((NIL <> LList) and TeamHasSkill( LList, T ))
				or HasTalent( TruePC, NAS_JackOfAll )
				)
			) then begin
{$ELSE PATCH_CHEAT}
		if ( SkillMan[ T ].Usage = USAGE_Clue ) and ( TeamHasSkill( GB , NAV_DefPlayerTeam , T ) or HasTalent( TruePC , NAS_JackOfAll ) ) then begin
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_I18N}
			msg := ReplaceHash( I18N_MsgString('BACKPACK','ClueSkillPrompt'), I18N_Name('SkillMan',SkillMan[ T ].Name), GearName(Item) );
{$ELSE PATCH_I18N}
			msg := ReplaceHash( MsgString( 'BACKPACK_ClueSkillPrompt' ) , SkillMan[ T ].Name );
			msg := ReplaceHash( msg , GearName( Item ) );
{$ENDIF PATCH_I18N}
			AddRPGMenuItem( SkMenu , msg , T );
		end;
	end;
{$IFDEF PATCH_I18N}
{$ELSE PATCH_I18N}
	RPMSortAlpha( SkMenu );
{$ENDIF PATCH_I18N}
	AddRPGMenuItem( SkMenu , MsgSTring( 'BACKPACK_CancelSkillUse' ) , -1 );

{$IFDEF SDLMODE}
	InfoGB := GB;
	BP_Source := TruePC;
	BP_Focus := Item;
	T := SelectMenu( SkMenu , @FocusOnOneItemRedraw);
{$ELSE}
  {$IFDEF PATCH_CHEAT}
	InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
	T := SelectMenu( SkMenu );
{$ENDIF}
	DisposeRPGMenu( SkMenu );

	if T <> -1 then begin
		UseScriptItem( GB , TruePC , Item , 'CLUE' + BStr( T ) );
	end;
end;

Procedure EatItem( GB: GameBoardPtr; TruePC , Item: GearPtr );
	{ The PC wants to eat this item. Give it a try. }
const
	WaitTime_Max = 32767;
var
	effect: String;
	WaitTime: Int64;
begin
{$IFDEF PATCH_GH}
	if (NIL = TruePC) or (TruePC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	TruePC := LocatePilot( TruePC );

{$IFDEF PATCH_GH}
	if (NIL = TruePC) or (TruePC^.G <= GG_DisposeGear) then begin
{$ELSE PATCH_GH}
	if TruePC = Nil then begin
{$ENDIF PATCH_GH}
		DialogMsg( ReplaceHash( MsgString( 'BACKPACK_CantBeEaten' ) , GearName( Item ) ) );

	end else if ( NAttValue( TruePC^.NA , NAG_Condition , NAS_Hunger ) > ( Item^.V div 2 ) ) or ( Item^.V = 0 ) then begin
		{ Show a message. }
{$IFDEF PATCH_I18N}
		DialogMsg( ReplaceHash( I18N_MsgString('BACKPACK','YouAreEating'), GearName(TruePC), GearName(Item) ) );
{$ELSE PATCH_I18N}
		DialogMsg( ReplaceHash( ReplaceHash( MsgString( 'BACKPACK_YouAreEating' ) , GearName( TruePC ) ) , GearName( Item ) ) );
{$ENDIF PATCH_I18N}

		{ Eating takes time... }
		WaitTime := ReactionTime( TruePC ) * GearMass( Item ) + 1;
		while (WaitTime_Max < WaitTime) do begin
			WaitAMinute( GB, TruePC, WaitTime_Max );
			WaitTime := WaitTime - WaitTime_Max;
		end;
		WaitAMinute( GB, TruePC, WaitTime );

		{ ...and also exits the backpack. }
		ForceQuit := True;

		{ Locate the PC's Character record, then adjust hunger values. }
		AddNAtt( TruePC^.NA , NAG_Condition , NAS_Hunger , -Item^.V );
		AddMoraleDmg( TruePC , -( Item^.Stat[ STAT_MoraleBoost ] * FOOD_MORALE_FACTOR ) );

		{ Invoke the item's effect, if any. }
		effect := SAttValue( Item^.SA , 'EFFECT' );
		if effect <> '' then begin
{$IFNDEF SDLMODE}
			GFCombatDisplay( GB );
{$ENDIF}
{$IFDEF PATCH_GH}
			EffectFrontEnd( GB , Item , TruePC , effect , '' );
{$ELSE PATCH_GH}
			EffectFrontEnd( GB , TruePC , effect , '' );
{$ENDIF PATCH_GH}
		end;

		{ Destroy the item, if appropriate. }
		Dec( Item^.Stat[ STAT_FoodQuantity ] );
		if Item^.Stat[ STAT_FoodQuantity ] < 1 then begin
			if IsInvCom( Item ) then begin
				RemoveGEar( Item^.Parent^.InvCom , Item );
			end else if IsSubCom( Item ) then begin
				RemoveGEar( Item^.Parent^.SubCom , Item );
			end;
		end;
	end else begin
		DialogMsg( MsgString( 'BACKPACK_NotHungry' ) );
	end;
end;


{$IFDEF PATCH_CHEAT}
Procedure ThisItemWasSelected( GB: GameBoardPtr; var LList: GearPtr; TruePC , PC , Item: GearPtr; ShowOnly: Boolean );
{$ELSE PATCH_CHEAT}
Procedure ThisItemWasSelected( GB: GameBoardPtr; var LList: GearPtr; TruePC , PC , Item: GearPtr );
{$ENDIF PATCH_CHEAT}
	{ TruePC is the primary character, who may be doing repairs }
	{  and stuff. }
	{ PC is the current master being examined, which may well be }
	{  a mecha belonging to the TruePC rather than the TruePC itself. }
	{ LList is a list of mecha and other things which may or may not }
	{  belong to the same team as TruePC et al. }
	{ Item is the piece of wargear currently being examined. }
var
	TIWS_Menu: RPGMenuPtr;
{$IFDEF PATCH_GH}
	SI,TI: Integer;
{$ENDIF PATCH_GH}
	N: Integer;
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the GG_DisposeGear of LList at here. }
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
	{if (NIL = TruePC) or (TruePC^.G <= GG_DisposeGear) then Exit;}
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	TIWS_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );

{$IFDEF PATCH_GH}
	SI := 1;
	TI := 1;
{$ELSE PATCH_GH}
{$IFDEF PATCH_CHEAT}
    if not(ShowOnly) then begin
{$ENDIF PATCH_CHEAT}
	if Item^.G = GG_Usable then AddRPGMenuItem( TIWS_Menu , ReplaceHash( MsgString( 'BACKPACK_UseItem' ) , GearName( Item ) ) , -9 );
	if Item^.G = GG_Consumable then AddRPGMenuItem( TIWS_Menu , ReplaceHash( MsgString( 'BACKPACK_EatItem' ) , GearName( Item ) ) , -10 );

	if SATtValue( Item^.SA , 'USE' ) <> '' then AddRPGMenuItem( TIWS_Menu , ReplaceHash( MsgString( 'BACKPACK_UseItemScript' ) , GearName( Item ) ) , -11 );

	if Item^.G = GG_Ammo then AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_LoadAmmo' ) , -5 );
	if IsInvCom( Item ) then begin
		if Item^.Parent = PC then begin
{$IFDEF PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('ThisItemWasSelected','Equip'), GearName(Item) ), -2 );
{$ELSE PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , 'Equip ' + GearName( Item ) , -2 );
{$ENDIF PATCH_I18N}
			if ( FindMaster( Item ) <> Nil ) and ( FindMaster( Item )^.G = GG_Mecha ) then begin
{$IFDEF PATCH_CHEAT}
				if (NIL <> TruePC) then begin
  {$IFDEF PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Install'), GearName(Item) ) , -8 );
  {$ELSE PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_Install' ) + GearName( Item ) , -8 );
  {$ENDIF PATCH_I18N}
					if 0 < SAttValueToInt(Item^.SA,SATT_TRANSFORMABLE) then begin
  {$IFDEF PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Transformation'), GearName(Item) ) , -13 );
  {$ELSE PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , MsgString( 'Transformation' ) + GearName( Item ) , -13 );
  {$ENDIF PATCH_I18N}
					end;
				end;
{$ELSE PATCH_CHEAT}
  {$IFDEF PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Install'), GearName(Item) ) , -8 );
  {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_Install' ) + GearName( Item ) , -8 );
  {$ENDIF PATCH_I18N}
{$ENDIF PATCH_CHEAT}
			end;
		end else begin
{$IFDEF PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('ThisItemWasSelected','Unequip'), GearName(Item) ), -3 );
{$ELSE PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , 'Unequip ' + GearName( Item ) , -3 );
{$ENDIF PATCH_I18N}
		end;
{$IFDEF PATCH_CHEAT}
		if (NIL <> LList) then begin
			AddRPGMenuItem( TIWS_Menu, MsgString('BACKPACK_TradeItem'), -6 );
		end;
{$ELSE PATCH_CHEAT}
		if ( LList <> Nil ) and ( GB <> Nil ) then AddRPGMenuItem ( TIWS_Menu , MsgString( 'BACKPACK_TradeItem' ) , -6 );
{$ENDIF PATCH_CHEAT}
		AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_DropItem' ) , -4 );
	end else if ( FindMaster( Item ) <> Nil ) and ( FindMaster( Item )^.G = GG_Mecha ) and CanBeExtracted( Item ) then begin
{$IFDEF PATCH_CHEAT}
		if (NIL <> TruePC) then
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_I18N}
		AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Remove'), GearName(Item) ) , -7 );
{$ELSE PATCH_I18N}
		AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_Remove' ) + GearName( Item ) , -7 );
{$ENDIF PATCH_I18N}
	end;
{$IFDEF PATCH_CHEAT}
	if not(IsInvCom(Item)) and (NIL = Item^.Parent) and (NIL <> TruePC) then begin
		if 0 < SAttValueToInt(Item^.SA,SATT_TRANSFORMABLE) then begin
  {$IFDEF PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Transformation'), GearName(Item) ) , -13 );
  {$ELSE PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , MsgString( 'Transformation' ) + GearName( Item ) , -13 );
  {$ENDIF PATCH_I18N}
		end;
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_CHEAT}
		if 0 < SAttValueToInt(Item^.SA,SATT_SEPARABLE) then begin
  {$IFDEF PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','PurgeParts'), GearName(Item) ) , -14 );
  {$ELSE PATCH_I18N}
			AddRPGMenuItem( TIWS_Menu , MsgString( 'PurgeParts' ) + GearName( Item ) , -14 );
  {$ENDIF PATCH_I18N}
		end;
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_CHEAT}
	end;
{$ENDIF PATCH_CHEAT}
	AddRepairOptions( TIWS_Menu , TruePC , Item );

	if ( Item^.G = GG_Weapon ) or ( ( Item^.G = GG_Ammo ) and ( Item^.S = GS_Grenade ) ) then begin
		if NAttValue( Item^.NA , NAG_WeaponModifier , NAS_SafetySwitch ) = 0 then begin
			AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_EngageSafety' ) , -12 );
		end else begin
			AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_DisengageSafety' ) , -12 );
		end;
	end;

	AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_UseSkillOnItem' ) , 1 );
{$IFDEF PATCH_CHEAT}
    end;
{$ENDIF PATCH_CHEAT}
	AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_ExitTIWS' ) , -1 );
{$IFDEF PATCH_CHEAT}
	if Cheat_ThisItemWasSelected_AddMenuKey then begin
		AlphaKeyMenu( TIWS_Menu );
	end;
{$ENDIF PATCH_CHEAT}
{$ENDIF PATCH_GH}

	repeat
{$IFDEF PATCH_GH}
		TIWS_Menu := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );

  {$IFDEF PATCH_CHEAT}
		if not(ShowOnly) then begin
  {$ENDIF PATCH_CHEAT}
			if Item^.G = GG_Usable then AddRPGMenuItem( TIWS_Menu , ReplaceHash( MsgString( 'BACKPACK_UseItem' ) , GearName( Item ) ) , -9 );
			if Item^.G = GG_Consumable then AddRPGMenuItem( TIWS_Menu , ReplaceHash( MsgString( 'BACKPACK_EatItem' ) , GearName( Item ) ) , -10 );

			if SATtValue( Item^.SA , 'USE' ) <> '' then AddRPGMenuItem( TIWS_Menu , ReplaceHash( MsgString( 'BACKPACK_UseItemScript' ) , GearName( Item ) ) , -11 );

			if Item^.G = GG_Ammo then AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_LoadAmmo' ) , -5 );
			if IsInvCom( Item ) then begin
				if Item^.Parent = PC then begin
  {$IFDEF PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('ThisItemWasSelected','Equip'), GearName(Item) ), -2 );
  {$ELSE PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , 'Equip ' + GearName( Item ) , -2 );
  {$ENDIF PATCH_I18N}
					if ( FindMaster( Item ) <> Nil ) and ( FindMaster( Item )^.G = GG_Mecha ) then begin
  {$IFDEF PATCH_CHEAT}
						if (NIL <> TruePC) then begin
    {$IFDEF PATCH_I18N}
							AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Install'), GearName(Item) ) , -8 );
    {$ELSE PATCH_I18N}
							AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_Install' ) + GearName( Item ) , -8 );
    {$ENDIF PATCH_I18N}
							if 0 < SAttValueToInt(Item^.SA,SATT_TRANSFORMABLE) then begin
    {$IFDEF PATCH_I18N}
								AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Transformation'), GearName(Item) ) , -13 );
    {$ELSE PATCH_I18N}
								AddRPGMenuItem( TIWS_Menu , MsgString( 'Transformation' ) + GearName( Item ) , -13 );
    {$ENDIF PATCH_I18N}
							end;
						end;
  {$ELSE PATCH_CHEAT}
    {$IFDEF PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Install'), GearName(Item) ) , -8 );
    {$ELSE PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_Install' ) + GearName( Item ) , -8 );
    {$ENDIF PATCH_I18N}
  {$ENDIF PATCH_CHEAT}
					end;
				end else begin
  {$IFDEF PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('ThisItemWasSelected','Unequip'), GearName(Item) ), -3 );
  {$ELSE PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , 'Unequip ' + GearName( Item ) , -3 );
  {$ENDIF PATCH_I18N}
				end;
  {$IFDEF PATCH_CHEAT}
				if (NIL <> LList) then begin
					if CheckAlongPath_DisallowTransfering( Item ) then begin
    {$IFDEF PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'YouCannotTransferIt' ) , 0 );
    {$ELSE PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , 'You cannot transfer it.' , 0 );
    {$ENDIF PATCH_I18N}
					end else begin
						AddRPGMenuItem( TIWS_Menu, MsgString('BACKPACK_TradeItem'), -6 );
					end;
				end;
  {$ELSE PATCH_CHEAT}
				if ( LList <> Nil ) and ( GB <> Nil ) and IsSafeArea( GB ) then begin
					if CheckAlongPath_DisallowTransfering( Item ) then begin
    {$IFDEF PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'YouCannotTransferIt' ) , 0 );
    {$ELSE PATCH_I18N}
						AddRPGMenuItem( TIWS_Menu , 'You cannot transfer it.' , 0 );
    {$ENDIF PATCH_I18N}
					end else begin
						AddRPGMenuItem ( TIWS_Menu , MsgString( 'BACKPACK_TradeItem' ) , -6 );
					end;
				end;
  {$ENDIF PATCH_CHEAT}
				if CheckAlongPath_DisallowDropping( Item ) then begin
					AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'YouCannotDropIt' ) , 0 );
				end else begin
					AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_DropItem' ) , -4 );
				end;
			end else if ( FindMaster( Item ) <> Nil ) and ( FindMaster( Item )^.G = GG_Mecha ) and CanBeExtracted( Item ) then begin
  {$IFDEF PATCH_CHEAT}
				if (NIL <> TruePC) then
  {$ENDIF PATCH_CHEAT}
  {$IFDEF PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Remove'), GearName(Item) ) , -7 );
  {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_Remove' ) + GearName( Item ) , -7 );
  {$ENDIF PATCH_I18N}
			end;
  {$IFDEF PATCH_CHEAT}
			if not(IsInvCom(Item)) and (NIL = Item^.Parent) and (NIL <> TruePC) then begin
				if 0 < SAttValueToInt(Item^.SA,SATT_TRANSFORMABLE) then begin
    {$IFDEF PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','Transformation'), GearName(Item) ) , -13 );
    {$ELSE PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , MsgString( 'Transformation' ) + GearName( Item ) , -13 );
    {$ENDIF PATCH_I18N}
				end;
  {$ENDIF PATCH_CHEAT}
  {$IFDEF PATCH_CHEAT}
				if 0 < SAttValueToInt(Item^.SA,SATT_SEPARABLE) then begin
    {$IFDEF PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , ReplaceHash( I18N_MsgString('BACKPACK','PurgeParts'), GearName(Item) ) , -14 );
    {$ELSE PATCH_I18N}
					AddRPGMenuItem( TIWS_Menu , MsgString( 'PurgeParts' ) + GearName( Item ) , -14 );
    {$ENDIF PATCH_I18N}
				end;
  {$ENDIF PATCH_CHEAT}
  {$IFDEF PATCH_CHEAT}
			end;
  {$ENDIF PATCH_CHEAT}
			AddRepairOptions( TIWS_Menu , TruePC , Item );

			if ( Item^.G = GG_Weapon ) or ( ( Item^.G = GG_Ammo ) and ( Item^.S = GS_Grenade ) ) then begin
				if NAttValue( Item^.NA , NAG_WeaponModifier , NAS_SafetySwitch ) = 0 then begin
					AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_EngageSafety' ) , -12 );
				end else begin
					AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_DisengageSafety' ) , -12 );
				end;
			end;

			AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_UseSkillOnItem' ) , 1 );
			if DisallowSelling( Item ) then begin
  {$IFDEF PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'AllowSelling' ) , 7 );
  {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , 'AllowSelling' , 7 );
  {$ENDIF PATCH_I18N}
			end else begin
  {$IFDEF PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'DisallowSelling' ) , 6 );
  {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , 'DisallowSelling' , 6 );
  {$ENDIF PATCH_I18N}
			end;
  {$IFDEF PATCH_CHEAT}
			if DisallowDropping( Item ) then begin
    {$IFDEF PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'AllowDropping' ) , 9 );
    {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , 'AllowDropping' , 9 );
    {$ENDIF PATCH_I18N}
			end else begin
    {$IFDEF PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'DisallowDropping' ) , 8 );
    {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , 'DisallowDropping' , 8 );
    {$ENDIF PATCH_I18N}
			end;
			if DisallowTransfering( Item ) then begin
    {$IFDEF PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'AllowTransfering' ) , 11 );
    {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , 'AllowTransfering' , 11 );
    {$ENDIF PATCH_I18N}
			end else begin
    {$IFDEF PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , I18N_MsgString( 'ThisItemWasSelected' , 'DisallowTransfering' ) , 10 );
    {$ELSE PATCH_I18N}
				AddRPGMenuItem( TIWS_Menu , 'DisallowTransfering' , 10 );
    {$ENDIF PATCH_I18N}
			end;
		end;
  {$ENDIF PATCH_CHEAT}
		AddRPGMenuItem( TIWS_Menu , MsgString( 'BACKPACK_ExitTIWS' ) , -1 );
  {$IFDEF PATCH_CHEAT}
		if Cheat_ThisItemWasSelected_AddMenuKey then begin
			AlphaKeyMenu( TIWS_Menu );
		end;
  {$ENDIF PATCH_CHEAT}
		TIWS_Menu^.SelectItem := SI;
		TIWS_Menu^.TopItem    := TI;
{$ENDIF PATCH_GH}
{$IFDEF SDLMODE}
		BP_Focus := Item;
		BP_Source := PC;
		InfoGB := GB;
		N := SelectMenu( TIWS_Menu , @FocusOnOneItemRedraw );
{$ELSE}
  {$IFDEF PATCH_CHEAT}
		DisplayGearInfo( Item , GB );
  {$ELSE PATCH_CHEAT}
		DisplayGearInfo( Item );
  {$ENDIF PATCH_CHEAT}
		N := SelectMenu( TIWS_Menu );
{$ENDIF}
{$IFDEF PATCH_GH}
		SI := TIWS_Menu^.SelectItem;
		TI := TIWS_Menu^.TopItem;
		DisposeRPGMenu( TIWS_Menu );
{$ENDIF PATCH_GH}
		if N > 100 then begin
			DoFieldRepair( GB , TruePC , Item , RepairSkillIndex[N-100] );
		end else begin
			case N of
{$IFDEF PATCH_CHEAT}
				1: UseSkillOnItem( GB, LList, TruePC, Item );
{$ELSE PATCH_CHEAT}
				1: UseSkillOnItem( GB , TruePC , Item );
{$ENDIF PATCH_CHEAT}
				-2: EquipItemFrontend( GB , PC , Item );
				-3: UnequipFrontEnd( GB , PC , Item );
				-4: DropFrontEnd( PC , Item );
				-5: InstallAmmoFrontEnd( GB , PC , Item );
				-6: TradeFrontEnd( GB , PC, Item , LList );
{$IFDEF PATCH_CHEAT}
				-7: ExtractFrontEnd( GB, LList, TruePC, PC, Item );
				-8: InstallFrontEnd( GB, LList, TruePC, PC, Item );
{$ELSE PATCH_CHEAT}
				-7: ExtractFrontEnd( GB , TruePC , PC , Item );
				-8: InstallFrontEnd( GB , TruePC , PC , Item );
{$ENDIF PATCH_CHEAT}
				-9: StartContinuousUseItem( GB , TruePC , Item );
				-10: EatItem( GB , PC , Item );
				-11: UseScriptItem( GB , TruePC , Item , 'USE' );
				-12: SetNAtt( Item^.NA , NAG_WeaponModifier , NAS_SafetySwitch , 1 - NAttValue( Item^.NA , NAG_WeaponModifier , NAS_SafetySwitch ) );
{$IFDEF PATCH_CHEAT}
				-13: UserTransformation( GB , Item , True );
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_CHEAT}
				-14: UserPurgeParts( GB , Item );
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_GH}
				6: SetSAtt( Item^.SA , TAG_DisallowSelling + ' <true>' );
				7: SetSAtt( Item^.SA , TAG_DisallowSelling + ' <>' );
				8: SetSAtt( Item^.SA , TAG_DisallowDropping + ' <true>' );
				9: SetSAtt( Item^.SA , TAG_DisallowDropping + ' <>' );
				10: SetSAtt( Item^.SA , TAG_DisallowTransfering + ' <true>' );
				11: SetSAtt( Item^.SA , TAG_DisallowTransfering + ' <>' );
{$ENDIF PATCH_GH}
			end;
		end;
{$IF DEFINED(PATCH_GH) and DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ForceQuit or ( Cheat_Avoid_Zombie and ( NIL <> GB ) and ( not GearOperational( TruePC ) ) );
{$ELSEIF DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ForceQuit or ( Cheat_Avoid_Zombie and ( not GearOperational( TruePC ) ) );
{$ELSE}
	until ( N < 0 ) or ForceQuit;
{$ENDIF}

{$IFDEF PATCH_GH}
{$ELSE PATCH_GH}
	DisposeRPGMenu( TIWS_Menu );
{$ENDIF PATCH_GH}
end;

{$IFDEF PATCH_CHEAT}
Procedure ThisItemWasSelected( GB: GameBoardPtr; var LList: GearPtr; TruePC , PC , Item: GearPtr );
begin
	ThisItemWasSelected( GB, LList, TruePC, PC, Item, False );
end;
{$ENDIF PATCH_CHEAT}

Function DoInvMenu( GB: GameBoardPtr; var LList: GearPtr; var PC,M: GearPtr ): Boolean;
	{ Return TRUE if the user selected Quit. }
var
{$IFDEF PATCH_GH}
	N: LongInt;
	OldPos: Integer;
{$ELSE PATCH_GH}
	N,OldPos: Integer;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	top, sel: Integer;
{$ENDIF PATCH_CHEAT}
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the GG_DisposeGear of LList at here. }
	{if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit(True);}
	if (NIL = M) or (M^.G <= GG_DisposeGear) then Exit(True);
{$ENDIF PATCH_GH}

{$IFDEF PATCH_CHEAT}
	top := 0;
	sel := 0;
{$ENDIF PATCH_CHEAT}
	Repeat
{$IFDEF SDLMODE}
		InfoGear := M;
		InfoGB := GB;
		BP_ActiveMenu := InvRPM;
		BP_SeekSibs := False;
		BP_Source := M;
{$ELSE}
  {$IFDEF PATCH_CHEAT}
		InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
{$IFDEF PATCH_CHEAT}
		if Cheat_InvMenu_KeepPosition and (0 < sel) then begin
			InvRPM^.TopItem := top;
			SetItemByPosition( InvRPM, sel );
		end;
{$ENDIF PATCH_CHEAT}
{$IFDEF SDLMODE}
		N := SelectMenu( INVRPM , @InvRedraw);
{$ELSE}
		N := SelectMenu( InvRPM );
{$ENDIF}
{$IFDEF PATCH_CHEAT}
		top := InvRPM^.TopItem;
		sel := InvRPM^.SelectItem;
{$ENDIF PATCH_CHEAT}

		{ If an item was selected, pass it along to the appropriate }
		{ procedure. }
		if N > 0 then begin
			OldPos := InvRPM^.selectitem;
{$IFDEF PATCH_CHEAT}
			if ' ' = RPMLocateByPosition( InvRPM, sel )^.msg[1] then begin
  {$IFDEF DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N, False, InvRPM_MaxNum, 'DoInvMenu' ), not Cheat_Enable_SubInvMenu );
  {$ELSE DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N ), not Cheat_Enable_SubInvMenu );
  {$ENDIF DEBUG}
			end else begin
  {$IFDEF DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N, False, InvRPM_MaxNum, 'DoInvMenu' ) );
  {$ELSE DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N ) );
  {$ENDIF DEBUG}
			end;
{$ELSE PATCH_CHEAT}
  {$IFDEF DEBUG}
			ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N, False, InvRPM_MaxNum, 'DoInvMenu' ) );
  {$ELSE DEBUG}
			ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N ) );
  {$ENDIF DEBUG}
{$ENDIF PATCH_CHEAT}
			{ Restore the display. }
			UpdateBackpack( M );
			SetItemByPosition( InvRPM, OldPos );
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
  {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
		end else if N = -3 then begin
			M := FindNextPC( GB, M, True );
			N := 0;
			{ Restore the display. }
			UpdateBackpack( M );
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
  {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
		end else if N = -4 then begin
			M := FindPrevPC( GB, M, True );
			N := 0;
			{ Restore the display. }
			UpdateBackpack( M );
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
  {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
{$IFDEF PATCH_GH}
		end else if ( -5 = N ) then begin
			DropAllFrontEnd( GB , LList , M , M^.InvCom );
			UpdateBackpack( M );
  {$IFNDEF SDLMODE}
    {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
    {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
    {$ENDIF PATCH_CHEAT}
  {$ENDIF}
			N := 0;
		end else if ( -6 = N ) then begin
			TradeAllFrontEnd( GB , M , M^.InvCom , LList );
			UpdateBackpack( M );
			N := 0;
{$ENDIF PATCH_GH}
		end;
{$IFDEF PATCH_CHEAT}
		if N = -128 then begin
  {$IFDEF DEBUG}
			SwapMenu( ZONE_InvMenu , LocateGearByNumber( M , RPMLocateByPosition( InvRPM , InvRPM^.SelectItem )^.value, False, InvRPM_MaxNum, 'DoInvMenu' ) );
  {$ELSE DEBUG}
			SwapMenu( ZONE_InvMenu , LocateGearByNumber( M , RPMLocateByPosition( InvRPM , InvRPM^.SelectItem )^.value ) );
  {$ENDIF DEBUG}
			UpdateBackpack( M );
  {$IFNDEF SDLMODE}
    {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
    {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
    {$ENDIF PATCH_CHEAT}
  {$ENDIF}
			N := 0;
		end;
{$ENDIF PATCH_CHEAT}
{$IF DEFINED(PATCH_GH) and DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ForceQuit or ( Cheat_Avoid_Zombie and ( NIL <> GB ) and ( not GearOperational( PC ) ) );
{$ELSEIF DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ForceQuit or ( Cheat_Avoid_Zombie and ( not GearOperational( PC ) ) );
{$ELSE}
	until ( N < 0 ) or ForceQuit;
{$ENDIF}

{$IFNDEF SDLMODE}
	DisplayMenu( InvRPM );
{$ENDIF}

	DoInvMenu := N=-1;
end;

Function DoEqpMenu( GB: GameBoardPtr; var LList: GearPtr; var PC,M: GearPtr ): Boolean;
	{ Return TRUE if the user selected Quit. }
var
{$IFDEF PATCH_GH}
	N: LongInt;
	OldPos: Integer;
{$ELSE PATCH_GH}
	N, OldPos: Integer;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	top, sel: Integer;
{$ENDIF PATCH_CHEAT}
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the GG_DisposeGear of LList at here. }
	{if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit(True);}
	if (NIL = M) or (M^.G <= GG_DisposeGear) then Exit(True);
{$ENDIF PATCH_GH}

{$IFDEF PATCH_CHEAT}
	top := 0;
	sel := 0;
{$ENDIF PATCH_CHEAT}
	Repeat
{$IFDEF SDLMODE}
		InfoGear := M;
		InfoGB := GB;
		BP_ActiveMenu := EqpRPM;
		BP_SeekSibs := False;
		BP_Source := M;
{$ELSE}
  {$IFDEF PATCH_CHEAT}
		InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
{$IFDEF PATCH_CHEAT}
		if Cheat_EqpMenu_KeepPosition and (0 < sel) then begin
			EqpRPM^.TopItem := top;
			SetItemByPosition( EqpRPM, sel );
		end;
{$ENDIF PATCH_CHEAT}
{$IFDEF SDLMODE}
		N := SelectMenu( EqpRPM , @EqpRedraw);
{$ELSE}
		N := SelectMenu( EqpRPM );
{$ENDIF}
{$IFDEF PATCH_CHEAT}
		top := EqpRPM^.TopItem;
		if 0 < N then begin
			SetItemByValue( EqpRPM, N );
		end;
		sel := EqpRPM^.SelectItem;
{$ENDIF PATCH_CHEAT}

		{ If an item was selected, pass it along to the appropriate }
		{ procedure. }
		if N > 0 then begin
			OldPos := EqpRPM^.selectitem;
{$IFDEF PATCH_CHEAT}
			if ' ' = RPMLocateByPosition( EqpRPM, sel )^.msg[1] then begin
  {$IFDEF DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N, False, EqpRPM_MaxNum, 'DoEqpMenu' ), True );
  {$ELSE DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N ), True );
  {$ENDIF DEBUG}
			end else begin
  {$IFDEF DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N, False, EqpRPM_MaxNum, 'DoEqpMenu' ) );
  {$ELSE DEBUG}
				ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N ) );
  {$ENDIF DEBUG}
			end;
{$ELSE PATCH_CHEAT}
  {$IFDEF DEBUG}
			ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N, False, EqpRPM_MaxNum, 'DoEqpMenu' ) );
  {$ELSE DEBUG}
			ThisItemWasSelected( GB , LList , PC , M , LocateGearByNumber( M , N ) );
  {$ENDIF DEBUG}
{$ENDIF PATCH_CHEAT}
			{ Restore the display. }
			UpdateBackpack( M );
			SetItemByPosition( EqpRPM, OldPos );
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
  {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
		end else if N = -3 then begin
			M := FindNextPC( GB, M, True );
			N := 0;
			{ Restore the display. }
			UpdateBackpack( M );
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
  {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
		end else if N = -4 then begin
			M := FindPrevPC( GB, M, True );
			N := 0;
			{ Restore the display. }
			UpdateBackpack( M );
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
  {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
{$IFDEF PATCH_GH}
		end else if ( -5 = N ) then begin
			DropEqpAllFrontEnd( GB , LList , M , M );
			UpdateBackpack( M );
  {$IFNDEF SDLMODE}
    {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( M , GB );
    {$ELSE PATCH_CHEAT}
			DisplayGearInfo( M );
    {$ENDIF PATCH_CHEAT}
  {$ENDIF}
			N := 0;
		end else if ( -6 = N ) then begin
			TradeEqpAllFrontEnd( GB , M , M , LList );
			UpdateBackpack( M );
			N := 0;
{$ENDIF PATCH_GH}
		end;
{$IF DEFINED(PATCH_GH) and DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ForceQuit or ( Cheat_Avoid_Zombie and ( NIL <> GB ) and ( not GearOperational( PC ) ) );
{$ELSEIF DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ForceQuit or ( Cheat_Avoid_Zombie and ( not GearOperational( PC ) ) );
{$ELSE}
	until ( N < 0 ) or ForceQuit;
{$ENDIF}

{$IFNDEF SDLMODE}
	DisplayMenu( EqpRPM );
{$ENDIF}

	DoEqpMenu := N=-1;
end;


Procedure RealBackpack( GB: GameBoardPtr; var LList: GearPtr; PC,M: GearPtr; StartWithInv: Boolean );
	{ This is the backpack routine which should allow the player to go }
	{ through all the stuff in his/her inventory, equip items, drop them, }
	{ reload weapons, and whatnot. It is based roughly upon the procedures }
	{ from DeadCold. }
var
	QuitBP: Boolean;
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the GG_DisposeGear of LList at here. }
	{if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;}
	if (NIL = M) or (M^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Set up the display. }
{$IFNDEF SDLMODE}
	DrawBPBorder;
{$ENDIF}
	ForceQuit := False;

	{ Initialize menus to NIL, then create them. }
	InvRPM := Nil;
	EqpRPM := Nil;
	UpdateBackpack( M );

	repeat
{$IFNDEF SDLMODE}
		GameMsg( MsgString( 'BACKPACK_Directions' ) , ZONE_Menu , MenuItem );
{$ENDIF}
		if StartWithInv then begin
			QuitBP := DoInvMenu( GB , LList , PC , M );
		end else begin
			QuitBP := DoEqpMenu( GB , LList , PC , M );
		end;

		{ If we have not been ordered to exit the loop, we must }
		{ have been ordered to switch menus. }
		StartWithInv := Not StartWithInv;
{$IF DEFINED(PATCH_GH) and DEFINED(PATCH_CHEAT)}
	until QuitBP or ForceQuit or ( Cheat_Avoid_Zombie and ( NIL <> GB ) and ( not ( GearOperational( PC ) ) ) );
{$ELSEIF DEFINED(PATCH_CHEAT)}
	until QuitBP or ForceQuit or ( Cheat_Avoid_Zombie and ( not GearOperational( PC ) ) );
{$ELSE}
	until QuitBP or ForceQuit;
{$ENDIF}

	DisposeRPGMenu( InvRPM );
	DisposeRPGMenu( EqpRPM );
end;

Procedure LancemateBackpack( GB: GameBoardPtr; PC,NPC: GearPtr );
	{ This is a header for the REALBACKPACK function. }
begin
	RealBackPack( GB , GB^.Meks , PC , NPC , True );
end;

Procedure BackpackMenu( GB: GameBoardPtr; PC: GearPtr; StartWithInv: Boolean );
	{ This is a header for the REALBACKPACK function. }
begin
{$IFDEF PATCH_GH}
	if (NIL = GB) then Exit;
{$ENDIF PATCH_GH}
	RealBackPack( GB , GB^.Meks , PC , PC , StartWithInv );
end;

{$IFDEF SDLMODE}
Procedure MPERedraw;
	{ Show Inventory, select Equipment. }
var
	N: Integer;
	Part: GearPtr;
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF DEBUG}
	MaxNum := 0;
	if ( NIL <> InfoGB ) then begin
		MaxNum := CountGearIndex( BP_Source, False, '' );
	end;
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
	if ( NIL <> InfoGB ) then begin
		SDLCombatDisplay( InfoGB );
	end;
	if (NIL <> BP_Source) and (GG_DisposeGear < BP_Source^.G) then begin
		InfoBox( ZONE_TargetInfo.GetRect() );
		DisplayTargetInfo( BP_Source, InfoGB, ZONE_TargetInfo );
	end;
{$ELSE PATCH_GH}
	SDLCombatDisplay( InfoGB );
{$ENDIF PATCH_GH}
	InfoBox( ZONE_FHQMenu1.GetRect() );
	InfoBox( ZONE_FHQMenu2.GetRect() );
	InfoBox( ZONE_FHQInfo.GetRect() );
	MechaEngineeringInfo( BP_Source, InfoGB, ZONE_FHQMenu1 );

	if ( BP_ActiveMenu <> Nil ) and ( BP_Source <> Nil ) then begin
		N := CurrentMenuItemValue( BP_ActiveMenu );
		if N > 0 then begin
{$IFDEF DEBUG}
			Part := LocateGearByNumber( BP_Source , N, False, MaxNum, 'PartBrowserRedraw' );
{$ELSE DEBUG}
			Part := LocateGearByNumber( BP_Source , N );
{$ENDIF DEBUG}
			if Part <> Nil then begin
				LongformGearInfo( Part , InfoGB, ZONE_FHQInfo );
			end else LongformGearInfo( BP_Source , InfoGB, ZONE_FHQInfo );
		end else LongformGearInfo( BP_Source , InfoGB, ZONE_FHQInfo );
	end;
end;
{$ENDIF}

Procedure MechaPartEditor( GB: GameBoardPtr; var LList: GearPtr; PC,Mek: GearPtr );
	{ This procedure may be used to browse through all the various }
	{ bits of a mecha and examine each one individually. }
var
	RPM: RPGMenuPtr;
{$IFDEF PATCH_CHEAT}
	N: LongInt;
	top,sel: Integer;
{$ELSE PATCH_CHEAT}
{$IFDEF PATCH_GH}
	N: LongInt;
	I: Integer;
{$ELSE PATCH_GH}
	N,I: Integer;
{$ENDIF PATCH_GH}
{$ENDIF PATCH_CHEAT}
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the GG_DisposeGear of LList at here. }
	{if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;}
	if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Set up the display. }
	DrawBPBorder;
{$IFDEF PATCH_CHEAT}
	top := 0;
	sel := 0;
{$ELSE PATCH_CHEAT}
	I := 0;
{$ENDIF PATCH_CHEAT}

	Repeat
{$IFDEF SDLMODE}
		RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_FHQMenu2 );
{$ELSE}
		RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_InvMenu );
{$ENDIF}
{$IFDEF DEBUG}
		MaxNum := BuildGearMenu( RPM , Mek );
{$ELSE DEBUG}
		BuildGearMenu( RPM , Mek );
{$ENDIF DEBUG}
{$IFDEF PATCH_CHEAT}
		if Cheat_MechaPartEditor_KeepPosition and (0 < sel) then begin
			RPM^.TopItem := top;
			SetItemByPosition( RPM, sel );
		end;
{$ELSE PATCH_CHEAT}
		if I > 0 then SetItemByPosition( RPM , I );
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_I18N}
		AddRPGMenuItem( RPM , I18N_MsgString('MechaPartEditor','Exit Editor') , -1 );
{$ELSE PATCH_I18N}
		AddRPGMenuItem( RPM , 'Exit Editor' , -1 );
{$ENDIF PATCH_I18N}
{$IFDEF PATCH_CHEAT}
		if Cheat_MenuOrder_Edit then begin
			AddRPGMenuKey( RPM , KeyMap[ KMC_EditMenuOrder ].KCode , -128 );
		end;
		if Cheat_MechaPartEditor_AddMenuKey then begin
			AlphaKeyMenu( RPM );
		end;
{$ENDIF PATCH_CHEAT}

{$IFNDEF SDLMODE}
		GameMsg( FullGearName( Mek ) + ' '  + MechaDescription( Mek ) , ZONE_EqpMenu , InfoGreen );
  {$IFDEF PATCH_CHEAT}
		DisplayGearInfo( Mek , GB );
  {$ELSE PATCH_CHEAT}
		DisplayGearInfo( Mek );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}
{$IFDEF SDLMODE}
		BP_Source := Mek;
		InfoGB := GB;
		BP_ActiveMenu := RPM;
		N := SelectMenu( RPM , @MPERedraw);
{$ELSE}
  {$IFDEF PATCH_CHEAT}
		InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
		N := SelectMenu( RPM );
{$ENDIF}
{$IFDEF PATCH_CHEAT}
		top := RPM^.TopItem;
		if 0 < N then begin
			SetItemByValue( RPM, N );
		end;
		sel := RPM^.SelectItem;
		if N = -128 then begin
  {$IFDEF SDLMODE}
    {$IFDEF DEBUG}
			InfoGear := LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.SelectItem )^.value, False, MaxNum, 'MechaPartEditor' );
    {$ELSE DEBUG}
			InfoGear := LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.SelectItem )^.value );
    {$ENDIF DEBUG}
  {$ENDIF SDLMODE}
  {$IFDEF DEBUG}
			SwapMenu( ZONE_InvMenu , LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.SelectItem )^.value, False, MaxNum, 'MechaPartEditor'  ) );
  {$ELSE DEBUG}
			SwapMenu( ZONE_InvMenu , LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.SelectItem )^.value ) );
  {$ENDIF DEBUG}
		end;
{$ELSE PATCH_CHEAT}
		I := RPM^.SelectItem;
{$ENDIF PATCH_CHEAT}
		DisposeRPGMenu( RPM );

		if N > -1 then begin
{$IFDEF DEBUG}
			ThisItemWasSelected( GB , LList , PC , Mek , LocateGearByNumber( Mek , N, False, MaxNum, 'MechaPartEditor' ) );
{$ELSE DEBUG}
			ThisItemWasSelected( GB , LList , PC , Mek , LocateGearByNumber( Mek , N ) );
{$ENDIF DEBUG}
		end;
{$IF DEFINED(PATCH_GH) and DEFINED(PATCH_CHEAT)}
	until ( N = -1 ) or ( Cheat_Avoid_Zombie and ( NIL <> GB ) and ( not GearOperational( PC ) ) );
{$ELSEIF DEFINED(PATCH_CHEAT)}
	until ( N = -1 ) or ( Cheat_Avoid_Zombie and ( not GearOperational( PC ) ) );
{$ELSE}
	until N = -1;
{$ENDIF}

end;

{$IFDEF SDLMODE}
Procedure PartBrowserRedraw;
	{ Redraw the screen for the part browser. }
var
	N: Integer;
	Part: GearPtr;
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}
begin
{$IFDEF DEBUG}
	MaxNum := 0;
	if ( NIL <> InfoGB ) then begin
		MaxNum := CountGearIndex( BP_Source, False, '' );
	end;
{$ENDIF DEBUG}
	if MPB_Redraw <> Nil then MPB_Redraw;
	if BPRD_CAPTION <> '' then begin
		InfoBox( ZONE_FHQTitle.GetRect() );
		CMessage( BPRD_CAPTION , ZONE_FHQTitle.GetRect() , InfoHilight );
	end;
	InfoBox( ZONE_FHQMenu.GetRect() );
	InfoBox( ZONE_FHQInfo.GetRect() );
	if ( BP_ActiveMenu <> Nil ) and ( BP_Source <> Nil ) then begin
		N := CurrentMenuItemValue( BP_ActiveMenu );
		if N > 0 then begin
{$IFDEF DEBUG}
			Part := LocateGearByNumber( BP_Source , N, False, MaxNum, 'PartBrowserRedraw' );
{$ELSE DEBUG}
			Part := LocateGearByNumber( BP_Source , N );
{$ENDIF DEBUG}
			if Part <> Nil then begin
				LongformGearInfo( Part , InfoGB, ZONE_FHQInfo );
			end else LongformGearInfo( BP_Source , InfoGB, ZONE_FHQInfo );
		end else LongformGearInfo( BP_Source , InfoGB, ZONE_FHQInfo );
	end;
end;
{$ENDIF}

{$IFDEF PATCH_GH}
  {$IFDEF SDLMODE}
Procedure MechaPartBrowser( Mek: GearPtr; RDP: RedrawProcedureType );
begin
	MechaPartBrowser( Mek, False, RDP );
end;
  {$ELSE SDLMODE}
Procedure MechaPartBrowser( Mek: GearPtr );
begin
	MechaPartBrowser( Mek, False );
end;
  {$ENDIF SDLMODE}
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
  {$IFDEF SDLMODE}
Procedure MechaPartBrowser( Mek: GearPtr; DebugMode: Boolean; RDP: RedrawProcedureType );
  {$ELSE SDLMODE}
Procedure MechaPartBrowser( Mek: GearPtr; DebugMode: Boolean );
  {$ENDIF SDLMODE}
{$ELSE PATCH_GH}
  {$IFDEF SDLMODE}
Procedure MechaPartBrowser( Mek: GearPtr; RDP: RedrawProcedureType );
  {$ELSE}
Procedure MechaPartBrowser( Mek: GearPtr );
  {$ENDIF}
{$ENDIF PATCH_GH}
	{ This procedure may be used to browse through all the various }
	{ bits of a mecha and examine each one individually. }
var
	RPM: RPGMenuPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
{$IFDEF PATCH_CHEAT}
	ReBrowse: Boolean;
{$ENDIF PATCH_CHEAT}
{$IFDEF PATCH_GH}
	top, sel: Integer;
{$ENDIF PATCH_GH}
{$IFDEF DEBUG}
	MaxNum: LongInt;
{$ENDIF DEBUG}

{$IFDEF PATCH_CHEAT}
	Key: String;

	Procedure MechaPartBrowser_SearchForward;
	var
		RPM_Item: RPGMenuItemPtr;
	begin
	{$IFDEF SDLMODE}
		Key := GetStringFromUser( 'Search String Forward', NIL, Key );
	{$ELSE SDLMODE}
		Key := GetStringFromUser( 'Search String Forward', Key );
	{$ENDIF SDLMODE}
		sel := RPM^.SelectItem;
		RPM_Item := RPMLocateByPosition( RPM, sel );
		repeat
			Inc( sel );
			RPM_Item := RPM_Item^.Next;
			if (RPM^.NumItem < sel) then begin
				sel := 1;
				RPM_Item := RPM^.FirstItem;
			end;
			if 0 < Pos( Key, RPM_Item^.msg ) then begin
				RPM^.SelectItem := sel;
				break;
			end;
		until sel = RPM^.SelectItem;
	end;

	Procedure MechaPartBrowser_SearchBackward;
	var
		RPM_Item: RPGMenuItemPtr;
	begin
	{$IFDEF SDLMODE}
		Key := GetStringFromUser( 'Search String Backward', NIL, Key );
	{$ELSE SDLMODE}
		Key := GetStringFromUser( 'Search String Backward', Key );
	{$ENDIF SDLMODE}
		sel := RPM^.SelectItem;
		repeat
			Dec( sel );
			if (sel < 1) then begin
				sel := RPM^.NumItem;
			end;
			RPM_Item := RPMLocateByPosition( RPM, sel );
			if 0 < Pos( Key, RPM_Item^.msg ) then begin
				RPM^.SelectItem := sel;
				break;
			end;
		until sel = RPM^.SelectItem;
	end;
{$ENDIF PATCH_CHEAT}
begin
{$IFDEF PATCH_GH}
	if not(DebugMode) then begin
		if (NIL = Mek) or (Mek^.G <= GG_DisposeGear) then Exit;
	end;
	top := 0;
	sel := 0;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_CHEAT}
	Key := '';

    repeat
	ReBrowse := False;
{$ENDIF PATCH_CHEAT}
{$IFDEF SDLMODE}
	MPB_Redraw := RDP;
	BP_Source := Mek;
	BPRD_Caption := ReplaceHash( MSgString('PARTBROWSER_CAPTION'), FullGearName( Mek ) );
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_FHQMenu );
	BP_ActiveMenu := RPM;
  {$IFDEF PATCH_I18N}
	DialogMsg( I18N_MsgString('MechaPartBrowser','Working on it') );
  {$ELSE PATCH_I18N}
	DialogMsg( 'Working on it' );
  {$ENDIF PATCH_I18N}
{$ELSE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_Menu );
{$ENDIF}
{$IFDEF PATCH_GH}
	MPB_Gear := Mek;
	if not(DebugMode) then begin
		if (NIL = MPB_Gear) or (MPB_Gear^.G <= GG_DisposeGear) then MPB_Gear := NIL;
	end;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
  {$IFDEF DEBUG}
	MaxNum := BuildGearMenu( RPM, Mek, DebugMode );
  {$ELSE DEBUG}
	BuildGearMenu( RPM, Mek, DebugMode );
  {$ENDIF DEBUG}
{$ELSE PATCH_GH}
  {$IFDEF DEBUG}
	MaxNum := BuildGearMenu( RPM , Mek );
  {$ELSE DEBUG}
	BuildGearMenu( RPM , Mek );
  {$ENDIF DEBUG}
{$ENDIF PATCH_GH}
{$IFDEF PATCH_I18N}
	AddRPGMenuItem( RPM , I18N_MsgString('MechaPartBrowser','Exit Browser') , -1 );
{$ELSE PATCH_I18N}
	AddRPGMenuItem( RPM , 'Exit Browser' , -1 );
{$ENDIF PATCH_I18N}
{$IFDEF PATCH_CHEAT}
	if Cheat_MenuOrder_Edit then begin
		AddRPGMenuKey( RPM , KeyMap[ KMC_EditMenuOrder ].KCode , -128 );
	end;
	if DebugMode and Cheat_MechaPartBrowser_Delete then begin
		AddRPGMenuKey( RPM , KeyMap[ KMC_Stop ].KCode , -3 );
	end;
	if DebugMode then begin
		AddRPGMenuKey( RPM , KeyMap[ KMC_ExamineMap ].KCode , -4 );
		AddRPGMenuKey( RPM , KeyMap[ KMC_Search ].KCode , -5 );
	end;
{$ENDIF PATCH_CHEAT}

{$IFDEF PATCH_GH}
	if 0 < sel then begin
		RPM^.TopItem := top;
		SetItemByPosition( RPM, sel );
	end;
{$ENDIF PATCH_GH}
	Repeat
{$IFDEF PATCH_GH}
  {$IFDEF SDLMODE}
		N := SelectMenu( RPM , @PartBrowserRedraw );
  {$ELSE SDLMODE}
    {$IFDEF PATCH_CHEAT}
		DisplayGearInfo( MPB_Gear, InfoGB , DebugMode );
    {$ELSE PATCH_CHEAT}
		DisplayGearInfo( MPB_Gear, DebugMode );
    {$ENDIF PATCH_CHEAT}
		N := SelectMenu( RPM );
  {$ENDIF SDLMODE}
{$ELSE PATCH_GH}
  {$IFDEF SDLMODE}
		N := SelectMenu( RPM , @PartBrowserRedraw );
  {$ELSE}
    {$IFDEF PATCH_CHEAT}
		DisplayGearInfo( Mek , InfoGB );
    {$ELSE PATCH_CHEAT}
		DisplayGearInfo( Mek );
    {$ENDIF PATCH_CHEAT}
		N := SelectMenu( RPM );
  {$ENDIF}
{$ENDIF PATCH_GH}

		if N > -1 then begin
{$IFDEF PATCH_GH}
  {$IFDEF DEBUG}
			MPB_Gear := LocateGearByNumber( Mek, N, DebugMode, MaxNum, 'MechaPartBrowser' );
  {$ELSE DEBUG}
			MPB_Gear := LocateGearByNumber( Mek, N, DebugMode );
  {$ENDIF DEBUG}
{$ELSE PATCH_GH}
{$IFDEF SDLMODE}
			N := -1;
{$ELSE}
  {$IFDEF PATCH_CHEAT}
			DisplayGearInfo( LocateGearByNumber( Mek , N ) , InfoGB );
  {$ELSE PATCH_CHEAT}
			DisplayGearInfo( LocateGearByNumber( Mek , N ) );
  {$ENDIF PATCH_CHEAT}
			EndOFGameMoreKey;
{$ENDIF}
{$ENDIF PATCH_GH}

{$IFDEF PATCH_CHEAT}
		end else if N = -128 then begin
  {$IFDEF PATCH_GH}
    {$IFDEF DEBUG}
			MPB_Gear := LocateGearByNumber( Mek, RPMLocateByPosition( RPM , RPM^.selectitem )^.value, DebugMode, MaxNum, 'MechaPartBrowser' );
    {$ELSE DEBUG}
			MPB_Gear := LocateGearByNumber( Mek, RPMLocateByPosition( RPM , RPM^.selectitem )^.value, DebugMode );
    {$ENDIF DEBUG}
  {$ELSE PATCH_GH}
    {$IFDEF SDLMODE}
      {$IFDEF DEBUG}
			MPB_Gear := LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.selectitem )^.value, False, MaxNum, 'MechaPartBrowser' );
      {$ELSE DEBUG}
			MPB_Gear := LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.selectitem )^.value );
      {$ENDIF DEBUG}
    {$ENDIF SDLMODE}
  {$ENDIF PATCH_GH}
  {$IFDEF PATCH_GH}
			if SwapMenu( ZONE_Menu, MPB_Gear ) then begin
  {$ELSE PATCH_GH}
    {$IFDEF DEBUG}
			if SwapMenu( ZONE_Menu , LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.selectitem )^.value, False, MaxNum, 'MechaPartBrowser' ) ) then begin
    {$ELSE DEBUG}
			if SwapMenu( ZONE_Menu , LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.selectitem )^.value ) ) then begin
    {$ENDIF DEBUG}
  {$ENDIF PATCH_GH}
				N := -1;
				ReBrowse := True;
			end;

		end else if N = -3 then begin
  {$IFDEF PATCH_GH}
    {$IFDEF DEBUG}
			MPB_Gear := LocateGearByNumber( Mek, RPMLocateByPosition( RPM , RPM^.selectitem )^.value, DebugMode, MaxNum, 'MechaPartBrowser' );
    {$ELSE DEBUG}
			MPB_Gear := LocateGearByNumber( Mek, RPMLocateByPosition( RPM , RPM^.selectitem )^.value, DebugMode );
    {$ENDIF DEBUG}
			Mark_GG_DisposeGear( MPB_Gear );
  {$ELSE PATCH_GH}
    {$IFDEF DEBUG}
			Mark_GG_DisposeGear( LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.selectitem )^.value, False, MaxNum, 'MechaPartBrowser' ) );
    {$ELSE DEBUG}
			Mark_GG_DisposeGear( LocateGearByNumber( Mek , RPMLocateByPosition( RPM , RPM^.selectitem )^.value ) );
    {$ENDIF DEBUG}
  {$ENDIF PATCH_GH}
			N := -1;
			ReBrowse := True;

		end else if N = -4 then begin
			MechaPartBrowser_SearchForward();
			N := -1;
			ReBrowse := True;
		end else if N = -5 then begin
			MechaPartBrowser_SearchBackward();
			N := -1;
			ReBrowse := True;
{$ENDIF PATCH_CHEAT}
		end;
	until N = -1;
{$IFDEF PATCH_GH}
	top := RPM^.TopItem;
	sel := RPM^.SelectItem;
{$ENDIF PATCH_GH}
	DisposeRPGMenu( RPM );
{$IFDEF PATCH_CHEAT}
    until False = ReBrowse;
{$ENDIF PATCH_CHEAT}
end;


{$IFDEF PATCH_GH}
Procedure FHQ_Transfer( GB: GameBoardPtr; var LList: GearPtr; PC,Item: GearPtr );
{$ELSE PATCH_GH}
Procedure FHQ_Transfer( var LList: GearPtr; PC,Item: GearPtr );
{$ENDIF PATCH_GH}
	{ An item has been selected. Allow it to be transferred to }
	{ one of the team's master gears. }
var
	RPM: RPGMenuPtr;
	M: GearPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
	Team: Integer;
{$ELSE PATCH_GH}
	N,Team: Integer;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the GG_DisposeGear of LList at here. }
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = Item) or (Item^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	{ Show the item's stats. }
{$IFNDEF SDLMODE}
  {$IFDEF PATCH_CHEAT}
	DisplayGearInfo( Item , GB );
  {$ELSE PATCH_CHEAT}
	DisplayGearInfo( Item );
  {$ENDIF PATCH_CHEAT}
{$ENDIF}

	{ Create the menu. }
{$IFDEF SDLMODE}
	RPM := CreateRPGMenu( MenuItem, MenuSelect, ZONE_FHQMenu );
{$ELSE}
	RPM := CreateRPGMenu( MenuItem, MenuSelect, ZONE_Menu );
{$ENDIF}
	M := LList;
	N := 1;
	Team := NAttValue( PC^.NA , NAG_LOcation , NAS_Team );
	while M <> Nil do begin
{$IFDEF PATCH_GH}
		if (GG_DisposeGear < M^.G) then begin
			if ( ( NAttValue( M^.NA , NAG_LOcation , NAS_Team ) = Team ) or ( NAttValue( M^.NA , NAG_LOcation , NAS_Team ) = NAV_LancemateTeam ) ) and IsMasterGear( M ) and IsLegalSlot( M , Item ) then begin
				AddRPGMenuItem( RPM , LanceMateMenuName( GB , M ) , N );
			end;
			Inc( N );
		end;
		M := M^.Next;
{$ELSE PATCH_GH}
		if ( ( NAttValue( M^.NA , NAG_LOcation , NAS_Team ) = Team ) or ( NAttValue( M^.NA , NAG_LOcation , NAS_Team ) = NAV_LancemateTeam ) ) and IsMasterGear( M ) and IsLegalSlot( M , Item ) then begin
			AddRPGMenuItem( RPM , LanceMateMenuName( M ) , N );
		end;

		M := M^.Next;
		Inc( N );
{$ENDIF PATCH_GH}
	end;

	{ Sort the menu, then add an exit option. }
	RPMSortAlpha( RPM );
	AddRPGMenuItem( RPM , MsgString( 'FHQ_ReturnToMain' ) , -1 );

	{ Get a menu selection, then exit the menu. }
	DialogMSG( MsgString( 'FHQ_SelectDestination' ) );
{$IFDEF SDLMODE}
	InfoGear := Item;
	BP_Source := LList;
	BP_ActiveMenu := RPM;
	BPRD_Caption := ReplaceHash( MsgString('TRANSFER_GiveTo'), GearName( Item ) );
	N := SelectMenu( RPM , @TransferRedraw );
{$ELSE}
	N := SelectMenu( RPM );
{$ENDIF}
	DisposeRPGMenu( RPM );

	if N > -1 then begin
		M := RetrieveGearSib( LList , N );
		DelinkGear( LList , Item );
		InsertInvCom( M , Item );
		DialogMSG( MsgString( 'FHQ_ItemMoved' ) );
	end else begin
		DialogMSG( MsgString( 'Cancelled' ) );
	end;
end;

Procedure Rename_Mecha( GB: GameBoardPtr; NPC: GearPtr );
	{ Enter a new name for NPC. }
var
	name: String;
{$IFDEF PATCH_GH}
	ask_msg: String;
	old_name: String;
	q: Boolean;
	canceled: Boolean;
{$ENDIF PATCH_GH}
begin
{$IFDEF PATCH_GH}
	if (NIL = NPC) or (NPC^.G <= GG_DisposeGear) then Exit;

  {$IFDEF SDLMODE}
	InfoGB := GB;
  {$ELSE SDLMODE}
    {$IFDEF PATCH_CHEAT}
	InfoGB := GB;
    {$ENDIF PATCH_CHEAT}
  {$ENDIF SDLMODE}

	q := False;
  {$IFDEF PATCH_I18N}
	old_name := SAttValue( NPC^.SA , 'name_rename' );
  {$ELSE PATCH_I18N}
	old_name := SAttValue( NPC^.SA , 'name' );
  {$ENDIF PATCH_I18N}
	name := GearName( NPC );
	ask_msg := ReplaceHash( MsgString( 'FHQ_Rename_Prompt' ) , GearName( NPC ) );
	repeat
  {$IFDEF SDLMODE}
		name := GetStringFromUser( ask_msg , @PlainRedraw , name , canceled );
  {$ELSE SDLMODE}
		name := GetStringFromUser( ask_msg , name , canceled );
		if ( NIL <> GB ) then begin
			GFCombatDisplay( GB );
		end;
  {$ENDIF SDLMODE}
		if canceled then begin
			q := True;
		end else if ( '' = name ) then begin
			q := True;
			Remove_Check_DuplicateNames( old_name );
  {$IFDEF PATCH_I18N}
			SetSAtt( NPC^.SA , 'name_rename <>' );
  {$ENDIF PATCH_I18N}
		end else if ( UpCase( old_name ) = UpCase( name ) ) then begin
			q := True;
  {$IFDEF PATCH_I18N}
		end else if ( UpCase( SAttValue( NPC^.SA , 'name' ) ) = UpCase( name ) ) then begin
			q := True;
			Remove_Check_DuplicateNames( old_name );
			SetSAtt( NPC^.SA , 'name_rename <>' );
  {$ENDIF PATCH_I18N}
		end else if not Check_DuplicateNames( name ) then begin
			q := True;
			Remove_Check_DuplicateNames( old_name );
			Set_Check_DuplicateNames( name );
  {$IFDEF PATCH_I18N}
			SetSAtt( NPC^.SA , 'name_rename <' + name + '>' );
  {$ELSE PATCH_I18N}
			SetSAtt( NPC^.SA , 'name <' + name + '>' );
  {$ENDIF PATCH_I18N}
		end else begin
			ask_msg := ReplaceHash( MsgString( 'FHQ_Rename_Prompt_Retry' ) , GearName( NPC ) );
		end;
	until q;
{$ELSE PATCH_GH}
  {$IFDEF SDLMODE}
	InfoGB := GB;
	name := GetStringFromUser( ReplaceHash( MsgString( 'FHQ_Rename_Prompt' ) , GearName( NPC ) ) , @PlainRedraw );
  {$ELSE SDLMODE}
    {$IFDEF PATCH_CHEAT}
	InfoGB := GB;
    {$ENDIF PATCH_CHEAT}
	name := GetStringFromUser( ReplaceHash( MsgString( 'FHQ_Rename_Prompt' ) , GearName( NPC ) ) );
	GFCombatDisplay( GB );
  {$ENDIF SDLMODE}
  {$IFDEF PATCH_I18N}
	SetSAtt( NPC^.SA , 'name_rename <' + name + '>' );
  {$ELSE PATCH_I18N}
	if name <> '' then SetSAtt( NPC^.SA , 'name <' + name + '>' );
  {$ENDIF PATCH_I18N}
{$ENDIF PATCH_GH}
end;

{$IFDEF PATCH_CHEAT}
Procedure Redesig_Mecha( GB: GameBoardPtr; NPC: GearPtr );
	{ Enter a new desig for NPC. }
var
	desig: String;
begin
  {$IFDEF PATCH_GH}
	if (NIL = NPC) or (NPC^.G <= GG_DisposeGear) then Exit;
  {$ENDIF PATCH_GH}

  {$IFDEF SDLMODE}
	InfoGB := GB;
    {$IFDEF PATCH_GH}
	desig := GetStringFromUser( ReplaceHash( MsgString( 'FHQ_Redesig_Prompt' ) , GearDesig( NPC ) ) , @PlainRedraw , GearDesig( NPC ) );
    {$ELSE PATCH_GH}
	desig := GetStringFromUser( ReplaceHash( MsgString( 'FHQ_Redesig_Prompt' ) , GearDesig( NPC ) ) , @PlainRedraw );
    {$ENDIF PATCH_GH}
  {$ELSE SDLMODE}
    {$IFDEF PATCH_CHEAT}
	InfoGB := GB;
    {$ENDIF PATCH_CHEAT}
    {$IFDEF PATCH_GH}
	desig := GetStringFromUser( ReplaceHash( MsgString( 'FHQ_Redesig_Prompt' ) , GearDesig( NPC ) ) , GearDesig( NPC ) );
	if (NIL <> GB) then begin
		GFCombatDisplay( GB );
	end;
    {$ELSE PATCH_GH}
	desig := GetStringFromUser( ReplaceHash( MsgString( 'FHQ_Redesig_Prompt' ) , GearDesig( NPC ) ) );
	GFCombatDisplay( GB );
    {$ENDIF PATCH_GH}
  {$ENDIF SDLMODE}
  {$IFDEF PATCH_I18N}
	SetSAtt( NPC^.SA , 'desig_redesig <' + desig + '>' );
  {$ELSE PATCH_I18N}
	if desig <> '' then SetSAtt( NPC^.SA , 'desig <' + desig + '>' );
  {$ENDIF PATCH_I18N}
end;
{$ENDIF PATCH_CHEAT}

Procedure FHQ_ThisWargearWasSelected( GB: GameBoardPtr; var LList: GearPtr; PC,M: GearPtr );
	{ A mecha has been selected by the PC from the FHQ main menu. }
	{ Offer up all the different choices of things the PC can }
	{ do with mecha - select pilot, repair, check inventory, etc. }
var
	RPM: RPGMenuPtr;
	N: Integer;
begin
{$IFDEF PATCH_GH}
	{ Don't kick out the GG_DisposeGear of LList at here. }
	if (NIL = PC) or (PC^.G <= GG_DisposeGear) then Exit;
	if (NIL = M) or (M^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	N := 1;
	repeat
		{ Show the mecha's stats. }
		{ Create the FHQ menu. }
{$IF DEFINED(PATCH_GH) and DEFINED(SDLMODE)}
		BPRD_Caption := '';
{$ENDIF DEFINED(PATCH_GH) and DEFINED(SDLMODE)}
{$IFDEF SDLMODE}
		RPM := CreateRPGMenu( MenuItem, MenuSelect, ZONE_FHQMenu );
{$ELSE}
  {$IFDEF PATCH_CHEAT}
		DisplayGearInfo( M , GB );
  {$ELSE PATCH_CHEAT}
		DisplayGearInfo( M );
  {$ENDIF PATCH_CHEAT}
		RPM := CreateRPGMenu( MenuItem, MenuSelect, ZONE_Menu );
		RPM^.Mode := RPMNoCleanup;
{$ENDIF}

		if IsMasterGear( M ) then begin
			if IsSafeArea( GB ) or OnTheMap( M ) then AddRPGMenuItem( RPM , MsgString( 'FHQ_GoBackpack' ) , 1 );
		end else if IsSafeArea( GB ) then begin
			AddRPGMenuItem( RPM , MsgString( 'FHQ_Transfer' ) , -3 );
		end;

		if IsSafeArea( GB ) then AddRepairOptions( RPM , PC , M );

		if M^.G = GG_Mecha then begin
			AddRPGMenuItem( RPM , MsgString( 'FHQ_SelectMecha' ) , 2 );
			AddRPGMenuItem( RPM , MsgString( 'FHQ_Rename' ) , 6 );
{$IFDEF PATCH_CHEAT}
			AddRPGMenuItem( RPM , MsgString( 'FHQ_Redesig' ) , 7 );
{$ENDIF PATCH_CHEAT}
		end;
		if IsSafeArea( GB ) then AddRPGMenuItem( RPM , MsgString( 'FHQ_PartEditor' ) , 4 );

{$IFDEF SDLMODE}
		if M^.G = GG_Mecha then AddRPGMenuItem( RPM , MsgString( 'FHQ_EditColor' ) , 5 );
{$ENDIF}

		AddRPGMenuItem( RPM , MsgString( 'FHQ_ReturnToMain' ) , -1 );
{$IFDEF PATCH_CHEAT}
		if Cheat_FieldHQ_AddMenuKey then begin
			AlphaKeyMenu( RPM );
		end;
{$ENDIF PATCH_CHEAT}

		SetItemByValue( RPM , N );

		{ Get a selection from the menu, then dispose of it. }
{$IFDEF SDLMODE}
		InfoGB := GB;
		infoGear := M;
		N := SelectMenu( RPM , @FHQWargearRedraw );
{$ELSE}
  {$IFDEF PATCH_CHEAT}
		InfoGB := GB;
  {$ENDIF PATCH_CHEAT}
		N := SelectMenu( RPM );
{$ENDIF}
		DisposeRPGMenu( RPM );

		if N > 100 then begin
			{ A repair option must have been selected. }
			DoFieldRepair( GB , PC , M , RepairSkillIndex[N-100] );

		end else begin
			case N of
				1: RealBackpack( GB , LList , PC , M , False );
				2: FHQ_SelectPilotForMecha( GB , M );
{$IFDEF PATCH_GH}
				-3: FHQ_Transfer( GB , LList , PC , M );
{$ELSE PATCH_GH}
				-3: FHQ_Transfer( LList , PC , M );
{$ENDIF PATCH_GH}
				4: MechaPartEditor( GB , LList , PC , M );
{$IFDEF SDLMODE}
				5: SelectColors( M , @PlainRedraw );
{$ENDIF}
				6: Rename_Mecha( GB , M );
{$IFDEF PATCH_CHEAT}
				7: Redesig_Mecha( GB , M );
{$ENDIF PATCH_CHEAT}
			end;

		end;

{$IF DEFINED(PATCH_GH) and DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ( Cheat_Avoid_Zombie and ( NIL <> GB ) and ( not GearOperational( PC ) ) );
{$ELSEIF DEFINED(PATCH_CHEAT)}
	until ( N < 0 ) or ( Cheat_Avoid_Zombie and ( not GearOperational( PC ) ) );
{$ELSE}
	until N < 0;
{$ENDIF}
{$IFDEF SDLMODE}
	CleanSpriteList;
{$ELSE}
	GFCombatDisplay( GB );
{$ENDIF}
end;

{$IFDEF PATCH_CHEAT}
  {$IFDEF SDLMODE}
Procedure CharViewRedraw;
var
	msg: String;
begin
	InfoBox( ZONE_FHQMenu.GetRect() );
	if ( NIL <> SP_GB ) and ( NIL <> SP_M ) then begin
		InfoBox( ZONE_FHQInfo.GetRect() );
		msg := Temporary_SelectItem^.desc;
		DrawSprite( ConfirmSprite( msg , TeamColorString( SP_GB , SP_M ) , 100 , 150 ) , ZONE_FHQInfo.GetRect() , 0 );
	end;
end;
  {$ENDIF}

Procedure SelectPortrait( GB: GameBoardPtr; M: GearPtr );
	{ The player wants to change the colors for sprite for this character. }
	{ The menu will be placed in the Menu area; assume the redrawer will }
	{ show whatever changes are made here. }
var
	RPM: RPGMenuPtr;
	fname: String;
begin
{$IFDEF PATCH_GH}
	if (NIL = M) or (M^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	SP_GB := GB;
	SP_M := M;

{$IFDEF SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_FHQMenu );
{$ELSE SDLMODE}
	RPM := CreateRPGMenu( MenuItem , MenuSelect , ZONE_Menu );
{$ENDIF SDLMODE}
{$IFDEF PATCH_CHEAT}
	if Cheat_Unlimited_Portrait then begin
		BuildFileMenu( RPM , Graphics_Directory + 'por_*.*' );
	end else begin
		if ( NAV_Male = NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) ) then begin
			BuildFileMenu( RPM , Graphics_Directory + 'por_m_*.*' );
		end else if ( NAV_Female = NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) ) then begin
			BuildFileMenu( RPM , Graphics_Directory + 'por_f_*.*' );
		end else begin
			BuildFileMenu( RPM , Graphics_Directory + 'por_m_*.*' );
			BuildFileMenu( RPM , Graphics_Directory + 'por_f_*.*' );
			BuildFileMenu( RPM , Graphics_Directory + 'por_n_*.*' );
		end;
	end;
{$ELSE PATCH_CHEAT}
	if ( NAV_Male = NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) ) then begin
		BuildFileMenu( RPM , Graphics_Directory + 'por_m_*.*' );
	end else if ( NAV_Female = NAttValue( M^.NA , NAG_CharDescription , NAS_Gender ) ) then begin
		BuildFileMenu( RPM , Graphics_Directory + 'por_f_*.*' );
	end else begin
		BuildFileMenu( RPM , Graphics_Directory + 'por_m_*.*' );
		BuildFileMenu( RPM , Graphics_Directory + 'por_f_*.*' );
		BuildFileMenu( RPM , Graphics_Directory + 'por_n_*.*' );
	end;
{$ENDIF PATCH_CHEAT}
	AddRPGMenuItem( RPM , MsgString( 'EXIT' ) , -1 );

{$IFDEF PATCH_GH}
	fname := SAttValue( M^.SA , 'SDL_PORTRAIT' );
	while (RPM^.SelectItem < RPM^.NumItem) do begin
		if RPMLocateByPosition(RPM,RPM^.SelectItem)^.msg = fname then begin
			break;
		end;
		Inc( RPM^.SelectItem );
	end;
	RPM^.TopItem := -1;
{$ENDIF PATCH_GH}

{$IFDEF SDLMODE}
	fname := SelectFile( RPM , @CharViewRedraw );
{$ELSE SDLMODE}
	fname := SelectFile( RPM );
{$ENDIF SDLMODE}

	if fname <> '' then begin
		SetSAtt( M^.SA , 'SDL_PORTRAIT <' + fname + '>' );
	end;

	DisposeRPGMenu( RPM );

	SP_GB := NIL;
	SP_M := NIL;
end;
{$ENDIF PATCH_CHEAT}

{$IFDEF PATCH_CHEAT}
Procedure BuildSiblingMenu( RPM: RPGMenuPtr; First , Item: GearPtr );
var
	N: Integer;
	Part: GearPtr;
begin
	{ Don't kick out the GG_DisposeGear of Item at here. }
	N := 0;
	Part := First;
	while Part <> Nil do begin
{$IFDEF PATCH_GH}
		if (GG_DisposeGear < Part^.G) then begin
			Inc(N);
			if (Part <> Item) then begin
				AddRPGMenuItem( RPM , '   ' + GearName( Part ) , N )
			end else begin
				AddRPGMenuItem( RPM , '< ' + GearName( Part ) , N );
			end;
		end;
		Part := Part^.Next;
{$ELSE PATCH_GH}
		Inc(N);
		if (Part <> Item) then AddRPGMenuItem( RPM , '   ' + GearName( Part ) , N )
		else AddRPGMenuItem( RPM , '< ' + GearName( Part ) , N );
		Part := Part^.Next;
{$ENDIF PATCH_GH}
	end;
end;

Procedure SwapParts( var F: GearPtr; A,B: GearPtr );
var
	t,pa,pb: GearPtr;
begin
{$IFDEF PATCH_GH}
	if (NIL = F) then Exit;
	if (NIL = A) or (A^.G <= GG_DisposeGear) then Exit;
	if (NIL = B) or (B^.G <= GG_DisposeGear) then Exit;
{$ENDIF PATCH_GH}

	t := F;
	while (t <> NIL) and (t <> A) and (t <> B) do begin
		t := t^.Next;
	end;
	if t = B then begin
		B := A;
		A := t;
	end;

	if F = A then begin
		pa := NIL;
	end else begin
		pa := F;
		while (pa^.Next <> A) do begin
			pa := pa^.next;
		end;
	end;

	pb := A;
	while (pb^.Next <> B) do begin
		pb := pb^.next;
	end;

	{ F ... pa A ... pb B ... => F ... pa B ... pb A ... }
	{ F ... pa A B ...        => F ... pa B A ...        }
	{ A ... pb B ...          => B ... pb A ...          }
	{ A B ...                 => B A ...                 }
	if A^.Next <> B then begin
		t := A^.Next;
	end else begin
		t := A;
	end;
	A^.Next := B^.Next;
	B^.Next := t;

	if pb <> A then begin
		pb^.Next := A;
	end;
	if (NIL = pa) then begin
		F := B;
	end else begin
		pa^.Next := B;
	end;
end;

{$IFDEF SDLMODE}
Function SwapMenu_NoParent( var FirstPart: GearPtr; Z: DynamicRect; Part: GearPtr ):Boolean;
{$ELSE SDLMODE}
Function SwapMenu_NoParent( var FirstPart: GearPtr; Z: Integer; Part: GearPtr ):Boolean;
{$ENDIF SDLMODE}
var
	done: Boolean;
	RPM: RPGMenuPtr;
	A,B: GearPtr;
{$IFDEF PATCH_GH}
	N: LongInt;
{$ELSE PATCH_GH}
	N: Integer;
{$ENDIF PATCH_GH}
	top: Integer = 1;
	sel: Integer = 1;
begin
	done := False;
	{ Don't kick out the GG_DisposeGear at here. }
	if (NIL <> FirstPart) and (NIL <> Part) then begin
		if 1 < NumSiblingGears(FirstPart) then begin
			RPM := CreateRPGMenu( MenuItem , MenuSelect , Z );
			A := Part;

			repeat
				BuildSiblingMenu( RPM, FirstPart, A );
				if done then AddRPGMenuItem( RPM , I18N_MsgString('BACKPACK_SwapMenu','Exit') , -1 )
				else AddRPGMenuItem( RPM , I18N_MsgString('BACKPACK_SwapMenu','Cancel') , -1 );
				RPM^.SelectItem := sel;
				RPM^.TopItem := top;
{$IFDEF SDLMODE}
				InfoBox( Z.GetRect() );
				N := SelectMenu( RPM , Nil );
{$ELSE SDLMODE}
				N := SelectMenu( RPM );
{$ENDIF SDLMODE}

				if 0 < N then begin
					B := RetrieveGearSib( FirstPart, N );
					if A = Nil then begin
						A := B;
					end else if A = B then begin
						A := Nil;
					end else begin
						SwapParts( FirstPart, A, B );
						done := True;
						A := Nil;
					end;

					sel := RPM^.SelectItem;
					top := RPM^.TopItem;
					ClearMenu( RPM );
				end;
			until N = -1;

			DisposeRPGMenu( RPM );
		end;
	end;
	SwapMenu_NoParent := done;
end;

{$IFDEF SDLMODE}
Function SwapMenu( Z: DynamicRect; Part: GearPtr ):Boolean;
{$ELSE SDLMODE}
Function SwapMenu( Z: Integer; Part: GearPtr ):Boolean;
{$ENDIF SDLMODE}
var
	done: Boolean;
begin
	done := False;
	{ Don't kick out the GG_DisposeGear at here. }
	if (NIL <> Part) and (NIL <> Part^.Parent) then begin
		if IsInvCom(Part) then begin
			done := SwapMenu_NoParent( Part^.Parent^.InvCom, Z, Part );
		end else begin
			done := SwapMenu_NoParent( Part^.Parent^.SubCom, Z, Part );
		end;
	end;
	SwapMenu := done;
end;
{$ENDIF PATCH_CHEAT}



initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: backpack.pp');
{$ENDIF DEBUG}
{$IFDEF PATCH_GH}
  {$IFDEF SDLMODE}
	InfoGear := NIL;
	InfoGB   := NIL;
	MPB_Redraw := NIL;
	MPB_Gear := NIL;
	SP_GB := NIL;
	SP_M := NIL;
	MPR_InvMenu     := NIL;
	MPR_InvMenuLGBN := NIL;
	MPR_InvMenuRGS  := NIL;
	Attach_SmartPointer( 'InfoGear: GearPtr', @InfoGear );
	Attach_SmartPointer( 'InfoGB: GameBoardPtr', @InfoGB );
	Attach_SmartPointer( 'MPB_Gear: GearPtr', @MPB_Gear );
  {$ELSE SDLMODE}
    {$IFDEF PATCH_CHEAT}
	InfoGB := NIL;
    {$ENDIF PATCH_CHEAT}
  {$ENDIF SDLMODE}
{$ENDIF PATCH_GH}
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: backpack.pp(finalization)');
{$ENDIF DEBUG}
end;

end.
